﻿namespace Controls
{
    using NodeList;
    using System;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Input;
    using System.Windows.Markup;

    public class ComboWithCheckBoxes : System.Windows.Controls.UserControl, IComponentConnector, IStyleConnector
    {
        private bool _contentLoaded;
        internal ComboBox CheckableCombo;
        public static readonly DependencyProperty DefaultTextProperty = DependencyProperty.Register("DefaultText", typeof(string), typeof(ComboWithCheckBoxes), new UIPropertyMetadata(string.Empty));
        private static int hilightedItem = -1;
        public static readonly DependencyProperty ItemsSourceProperty = DependencyProperty.Register("ItemsSource", typeof(object), typeof(ComboWithCheckBoxes), new UIPropertyMetadata(null));
        public static readonly DependencyProperty TextProperty = DependencyProperty.Register("Text", typeof(string), typeof(ComboWithCheckBoxes), new UIPropertyMetadata(string.Empty));
        internal ComboWithCheckBoxes UserControl;

        public ComboWithCheckBoxes()
        {
            this.InitializeComponent();
        }

        private void CheckBox_Click(object sender, RoutedEventArgs e)
        {
            this.SetText();
        }

        private void CheckBox_MouseEnter_1(object sender, MouseEventArgs e)
        {
            CheckBox box = (CheckBox) sender;
            ObservableNodeList itemsSource = (ObservableNodeList) this.ItemsSource;
            int num = 0;
            foreach (Node node in itemsSource)
            {
                if (node.user.Name.Equals(box.Content.ToString()))
                {
                    hilightedItem = num;
                    box.ToolTip = node.GroupMembers;
                }
                num++;
            }
        }

        private void CheckBox_MouseLeave(object sender, MouseEventArgs e)
        {
            hilightedItem = -1;
        }

        [DebuggerNonUserCode]
        public void InitializeComponent()
        {
            if (!this._contentLoaded)
            {
                this._contentLoaded = true;
                Uri resourceLocator = new Uri("/NAVEnhancedNotesCustomControls;component/combowithcheckboxes.xaml", UriKind.Relative);
                Application.LoadComponent(this, resourceLocator);
            }
        }

        private void SetText()
        {
            this.Text = (this.ItemsSource != null) ? this.ItemsSource.ToString() : this.DefaultText;
            if (string.IsNullOrEmpty(this.Text))
            {
                this.Text = this.DefaultText;
            }
        }

        [DebuggerNonUserCode, EditorBrowsable(EditorBrowsableState.Never)]
        void IComponentConnector.Connect(int connectionId, object target)
        {
            switch (connectionId)
            {
                case 1:
                    this.UserControl = (ComboWithCheckBoxes) target;
                    break;

                case 2:
                    this.CheckableCombo = (ComboBox) target;
                    break;

                default:
                    this._contentLoaded = true;
                    break;
            }
        }

        [EditorBrowsable(EditorBrowsableState.Never), DebuggerNonUserCode]
        void IStyleConnector.Connect(int connectionId, object target)
        {
            if (connectionId == 3)
            {
                ((CheckBox) target).Click += new RoutedEventHandler(this.CheckBox_Click);
                ((CheckBox) target).MouseEnter += new MouseEventHandler(this.CheckBox_MouseEnter_1);
                ((CheckBox) target).MouseLeave += new MouseEventHandler(this.CheckBox_MouseLeave);
            }
        }

        public string DefaultText
        {
            get
            {
                return (string) base.GetValue(DefaultTextProperty);
            }
            set
            {
                base.SetValue(DefaultTextProperty, value);
            }
        }

        public object ItemsSource
        {
            get
            {
                return base.GetValue(ItemsSourceProperty);
            }
            set
            {
                base.SetValue(ItemsSourceProperty, value);
                this.SetText();
            }
        }

        public string Text
        {
            get
            {
                return (string) base.GetValue(TextProperty);
            }
            set
            {
                base.SetValue(TextProperty, value);
            }
        }
    }
}

