﻿namespace Controls
{
    using RecordLink;
    using System;
    using System.Collections;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Input;
    using System.Windows.Markup;

    public class NotesListBox : UserControl, IComponentConnector
    {
        private bool _contentLoaded;
        private string _myEmail;
        internal ListBox listBox1;

        public NotesListBox()
        {
            this.InitializeComponent();
            this.InitCustomVariables();
        }

        public void Add(NAVRecordLink link)
        {
            this.Add(link, this.listBox1.Items.Count);
        }

        public void Add(NAVRecordLink link, int index)
        {
            if (this.IsNewRecord(link))
            {
                this.Clear();
            }
            if (!this.LinkExists(link))
            {
                if (this.ParentExists(link))
                {
                    ItemCollection items = this.listBox1.Items;
                    int num = -1;
                    foreach (object obj2 in (IEnumerable) items)
                    {
                        num++;
                        NotesTextBlock removeItem = (NotesTextBlock) obj2;
                        if (removeItem != null)
                        {
                            NAVRecordLink link2 = removeItem.link;
                            if ((link2 != null) && link.parentLinkID.Equals(link2.parentLinkID))
                            {
                                NotesTextBlock insertItem = this.CreateItem(link2);
                                NAVRecordLink link1 = insertItem.link;
                                link1.to = link1.to + ", " + link.to;
                                insertItem.link.AddChild(link.linkID);
                                this.listBox1.Items.Remove(removeItem);
                                if (index > 0)
                                {
                                    index--;
                                }
                                this.listBox1.Items.Insert(index, insertItem);
                                break;
                            }
                        }
                    }
                }
                else
                {
                    if (this.listBox1.Items.Count < index)
                    {
                        index = this.listBox1.Items.Count;
                    }
                    this.listBox1.Items.Insert(index, this.CreateItem(link));
                }
            }
        }

        public void Clear()
        {
            this.listBox1.Items.Clear();
            this.InitCustomVariables();
        }

        private NotesTextBlock CreateItem(NAVRecordLink link)
        {
            NotesTextBlock block = new NotesTextBlock {
                MyEmail = this._myEmail,
                link = link,
                MinWidth = base.MinWidth,
                MaxWidth = base.MaxWidth,
                Width = base.Width
            };
            block.CreateSmallNote();
            return block;
        }

        private NotesTextBlock CreateUnfocusableItem()
        {
            NotesTextBlock block = this.CreateItem(null);
            block.Focusable = false;
            block.Height = 50.0;
            return block;
        }

        private void InitCustomVariables()
        {
        }

        [DebuggerNonUserCode]
        public void InitializeComponent()
        {
            if (!this._contentLoaded)
            {
                this._contentLoaded = true;
                Uri resourceLocator = new Uri("/NAVEnhancedNotesCustomControls;component/noteslistbox.xaml", UriKind.Relative);
                Application.LoadComponent(this, resourceLocator);
            }
        }

        public bool IsNewRecord(NAVRecordLink link)
        {
            try
            {
                ItemCollection items = this.listBox1.Items;
                if (items == null)
                {
                    return true;
                }
                NotesTextBlock block = (NotesTextBlock) items[0];
                if (block != null)
                {
                    NAVRecordLink link2 = block.link;
                    if ((link2 != null) && !link.page.Equals(link2.page))
                    {
                        return true;
                    }
                }
            }
            catch (Exception)
            {
                return true;
            }
            return false;
        }

        public bool LinkExists(NAVRecordLink link)
        {
            ItemCollection items = this.listBox1.Items;
            foreach (object obj2 in (IEnumerable) items)
            {
                NotesTextBlock block = (NotesTextBlock) obj2;
                if (block != null)
                {
                    NAVRecordLink link2 = block.link;
                    if (link2 != null)
                    {
                        if (link.linkID.Equals(link2.linkID))
                        {
                            return true;
                        }
                        if (link2.children.Contains(link.linkID))
                        {
                            return true;
                        }
                    }
                }
            }
            return false;
        }

        public bool ParentExists(NAVRecordLink link)
        {
            ItemCollection items = this.listBox1.Items;
            foreach (object obj2 in (IEnumerable) items)
            {
                NotesTextBlock block = (NotesTextBlock) obj2;
                if (block != null)
                {
                    NAVRecordLink link2 = block.link;
                    if ((link2 != null) && link.parentLinkID.Equals(link2.parentLinkID))
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        public void SetSize(double w, double h)
        {
            w -= 13.0;
            this.listBox1.Width = w;
            this.listBox1.Height = h;
            base.Width = w;
            base.Height = h;
            ItemCollection items = this.listBox1.Items;
            foreach (object obj2 in (IEnumerable) items)
            {
                ((NotesTextBlock) obj2).SetSize(w, h);
            }
        }

        [DebuggerNonUserCode, EditorBrowsable(EditorBrowsableState.Never)]
        void IComponentConnector.Connect(int connectionId, object target)
        {
            switch (connectionId)
            {
                case 1:
                    ((NotesListBox) target).MouseLeftButtonDown += new MouseButtonEventHandler(this.UserControl_MouseLeftButtonDown);
                    ((NotesListBox) target).MouseUp += new MouseButtonEventHandler(this.UserControl_MouseUp);
                    break;

                case 2:
                    this.listBox1 = (ListBox) target;
                    break;

                default:
                    this._contentLoaded = true;
                    break;
            }
        }

        private void UserControl_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            NotesTextBlock selectedItem = (NotesTextBlock) this.listBox1.SelectedItem;
            if ((selectedItem != null) && (selectedItem.link != null))
            {
            }
        }

        private void UserControl_MouseUp(object sender, MouseButtonEventArgs e)
        {
            NotesTextBlock selectedItem = (NotesTextBlock) this.listBox1.SelectedItem;
            if ((selectedItem != null) && (selectedItem.link != null))
            {
            }
        }

        public string MyEmail
        {
            get
            {
                return this._myEmail;
            }
            set
            {
                this._myEmail = value;
            }
        }
    }
}

