﻿namespace Controls
{
    using System;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;
    using System.Windows.Documents;
    using System.Windows.Threading;

    public class RichTextBox : System.Windows.Controls.RichTextBox
    {
        private ITextFormatter _textFormatter;
        private bool isInvokePending;
        public static readonly DependencyProperty TextProperty = DependencyProperty.Register("Text", typeof(string), typeof(Controls.RichTextBox), new FrameworkPropertyMetadata(string.Empty, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault, new PropertyChangedCallback(Controls.RichTextBox.OnTextPropertyChanged), new CoerceValueCallback(Controls.RichTextBox.CoerceTextProperty), true, UpdateSourceTrigger.LostFocus));

        public RichTextBox()
        {
            base.Loaded += new RoutedEventHandler(this.RichTextBox_Loaded);
            base.IsReadOnly = true;
            base.Focusable = false;
        }

        public RichTextBox(FlowDocument document) : base(document)
        {
        }

        private static object CoerceTextProperty(DependencyObject d, object value)
        {
            return (value ?? "");
        }

        private void InvokeUpdateText()
        {
            if (!this.isInvokePending)
            {
                base.Dispatcher.BeginInvoke(DispatcherPriority.Background, new Action(this.UpdateText));
                this.isInvokePending = true;
            }
        }

        private static void OnTextPropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            Controls.RichTextBox box = (Controls.RichTextBox) d;
            string newValue = (string) e.NewValue;
            if (!newValue.Equals(""))
            {
                box.TextFormatter.SetText(box.Document, (string) e.NewValue);
            }
        }

        private void RichTextBox_Loaded(object sender, RoutedEventArgs e)
        {
            RoutedEventHandler handler = null;
            TextChangedEventHandler handler2 = null;
            Binding binding = BindingOperations.GetBinding(this, TextProperty);
            if (binding != null)
            {
                if ((binding.UpdateSourceTrigger == UpdateSourceTrigger.Default) || (binding.UpdateSourceTrigger == UpdateSourceTrigger.LostFocus))
                {
                    if (handler == null)
                    {
                        handler = (o, ea) => this.UpdateText();
                    }
                    base.LostFocus += handler;
                }
                else
                {
                    if (handler2 == null)
                    {
                        handler2 = (o, ea) => this.InvokeUpdateText();
                    }
                    base.TextChanged += handler2;
                }
            }
        }

        private void UpdateText()
        {
            this.Text = this.TextFormatter.GetText(base.Document);
            this.isInvokePending = false;
        }

        public string Text
        {
            get
            {
                return (string) base.GetValue(TextProperty);
            }
            set
            {
                base.SetValue(TextProperty, value);
            }
        }

        public ITextFormatter TextFormatter
        {
            get
            {
                if (this._textFormatter == null)
                {
                    this._textFormatter = new RtfFormatter();
                }
                return this._textFormatter;
            }
            set
            {
                this._textFormatter = value;
            }
        }
    }
}

