﻿namespace HTMLConverter
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Text;
    using System.Xml;

    internal class CssStylesheet
    {
        private List<StyleDefinition> _styleDefinitions;

        public CssStylesheet(XmlElement htmlElement)
        {
            if (htmlElement != null)
            {
                this.DiscoverStyleDefinitions(htmlElement);
            }
        }

        public void AddStyleDefinition(string selector, string definition)
        {
            selector = selector.Trim().ToLower();
            definition = definition.Trim().ToLower();
            if ((selector.Length != 0) && (definition.Length != 0))
            {
                if (this._styleDefinitions == null)
                {
                    this._styleDefinitions = new List<StyleDefinition>();
                }
                string[] strArray = selector.Split(new char[] { ',' });
                for (int i = 0; i < strArray.Length; i++)
                {
                    string str = strArray[i].Trim();
                    if (str.Length > 0)
                    {
                        this._styleDefinitions.Add(new StyleDefinition(str, definition));
                    }
                }
            }
        }

        public void DiscoverStyleDefinitions(XmlElement htmlElement)
        {
            if (htmlElement.LocalName.ToLower() != "link")
            {
                XmlNode node;
                if (htmlElement.LocalName.ToLower() != "style")
                {
                    for (node = htmlElement.FirstChild; node != null; node = node.NextSibling)
                    {
                        if (node is XmlElement)
                        {
                            this.DiscoverStyleDefinitions((XmlElement) node);
                        }
                    }
                }
                else
                {
                    StringBuilder builder = new StringBuilder();
                    for (node = htmlElement.FirstChild; node != null; node = node.NextSibling)
                    {
                        if ((node is XmlText) || (node is XmlComment))
                        {
                            builder.Append(this.RemoveComments(node.Value));
                        }
                    }
                    int num = 0;
                    while (num < builder.Length)
                    {
                        int startIndex = num;
                        while ((num < builder.Length) && (builder[num] != '{'))
                        {
                            if (builder[num] == '@')
                            {
                                while ((num < builder.Length) && (builder[num] != ';'))
                                {
                                    num++;
                                }
                                startIndex = num + 1;
                            }
                            num++;
                        }
                        if (num < builder.Length)
                        {
                            int num3 = num;
                            while ((num < builder.Length) && (builder[num] != '}'))
                            {
                                num++;
                            }
                            if ((num - num3) > 2)
                            {
                                this.AddStyleDefinition(builder.ToString(startIndex, num3 - startIndex), builder.ToString(num3 + 1, (num - num3) - 2));
                            }
                            if (num < builder.Length)
                            {
                                Debug.Assert(builder[num] == '}');
                                num++;
                            }
                        }
                    }
                }
            }
        }

        public string GetStyle(string elementName, List<XmlElement> sourceContext)
        {
            Debug.Assert(sourceContext.Count > 0);
            Debug.Assert(elementName == sourceContext[sourceContext.Count - 1].LocalName);
            if (this._styleDefinitions != null)
            {
                for (int i = this._styleDefinitions.Count - 1; i >= 0; i--)
                {
                    string[] strArray = this._styleDefinitions[i].Selector.Split(new char[] { ' ' });
                    int index = strArray.Length - 1;
                    int num3 = sourceContext.Count - 1;
                    string selectorLevel = strArray[index].Trim();
                    if (this.MatchSelectorLevel(selectorLevel, sourceContext[sourceContext.Count - 1]))
                    {
                        return this._styleDefinitions[i].Definition;
                    }
                }
            }
            return null;
        }

        private bool MatchSelectorLevel(string selectorLevel, XmlElement xmlElement)
        {
            if (selectorLevel.Length == 0)
            {
                return false;
            }
            int index = selectorLevel.IndexOf('.');
            int length = selectorLevel.IndexOf('#');
            string str = null;
            string str2 = null;
            string str3 = null;
            if (index >= 0)
            {
                if (index > 0)
                {
                    str3 = selectorLevel.Substring(0, index);
                }
                str = selectorLevel.Substring(index + 1);
            }
            else if (length >= 0)
            {
                if (length > 0)
                {
                    str3 = selectorLevel.Substring(0, length);
                }
                str2 = selectorLevel.Substring(length + 1);
            }
            else
            {
                str3 = selectorLevel;
            }
            if ((str3 != null) && (str3 != xmlElement.LocalName))
            {
                return false;
            }
            if ((str2 != null) && (HtmlToXamlConverter.GetAttribute(xmlElement, "id") != str2))
            {
                return false;
            }
            if ((str != null) && (HtmlToXamlConverter.GetAttribute(xmlElement, "class") != str))
            {
                return false;
            }
            return true;
        }

        private string RemoveComments(string text)
        {
            int index = text.IndexOf("/*");
            if (index < 0)
            {
                return text;
            }
            int num2 = text.IndexOf("*/", (int) (index + 2));
            if (num2 < 0)
            {
                return text.Substring(0, index);
            }
            return (text.Substring(0, index) + " " + this.RemoveComments(text.Substring(num2 + 2)));
        }

        private class StyleDefinition
        {
            public string Definition;
            public string Selector;

            public StyleDefinition(string selector, string definition)
            {
                this.Selector = selector;
                this.Definition = definition;
            }
        }
    }
}

