﻿namespace HTMLConverter
{
    using System;
    using System.Collections;
    using System.Diagnostics;

    internal class HtmlSchema
    {
        private static ArrayList _htmlBlockElements;
        private static Hashtable _htmlCharacterEntities;
        private static ArrayList _htmlElementsClosingColgroup;
        private static ArrayList _htmlElementsClosingDD;
        private static ArrayList _htmlElementsClosingDT;
        private static ArrayList _htmlElementsClosingLI;
        private static ArrayList _htmlElementsClosingOnParentElementEnd;
        private static ArrayList _htmlElementsClosingTbody;
        private static ArrayList _htmlElementsClosingTD;
        private static ArrayList _htmlElementsClosingTfoot;
        private static ArrayList _htmlElementsClosingTH;
        private static ArrayList _htmlElementsClosingThead;
        private static ArrayList _htmlElementsClosingTR;
        private static ArrayList _htmlEmptyElements;
        private static ArrayList _htmlInlineElements;
        private static ArrayList _htmlOtherOpenableElements;

        static HtmlSchema()
        {
            InitializeInlineElements();
            InitializeBlockElements();
            InitializeOtherOpenableElements();
            InitializeEmptyElements();
            InitializeElementsClosingOnParentElementEnd();
            InitializeElementsClosingOnNewElementStart();
            InitializeHtmlCharacterEntities();
        }

        internal static bool ClosesOnNextElementStart(string currentElementName, string nextElementName)
        {
            Debug.Assert(currentElementName == currentElementName.ToLower());
            switch (currentElementName)
            {
                case "colgroup":
                    return (_htmlElementsClosingColgroup.Contains(nextElementName) && IsBlockElement(nextElementName));

                case "dd":
                    return (_htmlElementsClosingDD.Contains(nextElementName) && IsBlockElement(nextElementName));

                case "dt":
                    return (_htmlElementsClosingDT.Contains(nextElementName) && IsBlockElement(nextElementName));

                case "li":
                    return _htmlElementsClosingLI.Contains(nextElementName);

                case "p":
                    return IsBlockElement(nextElementName);

                case "tbody":
                    return _htmlElementsClosingTbody.Contains(nextElementName);

                case "tfoot":
                    return _htmlElementsClosingTfoot.Contains(nextElementName);

                case "thead":
                    return _htmlElementsClosingThead.Contains(nextElementName);

                case "tr":
                    return _htmlElementsClosingTR.Contains(nextElementName);

                case "td":
                    return _htmlElementsClosingTD.Contains(nextElementName);

                case "th":
                    return _htmlElementsClosingTH.Contains(nextElementName);
            }
            return false;
        }

        internal static bool ClosesOnParentElementEnd(string xmlElementName)
        {
            return _htmlElementsClosingOnParentElementEnd.Contains(xmlElementName.ToLower());
        }

        internal static char EntityCharacterValue(string entityName)
        {
            if (_htmlCharacterEntities.Contains(entityName))
            {
                return (char) _htmlCharacterEntities[entityName];
            }
            return '\0';
        }

        private static void InitializeBlockElements()
        {
            _htmlBlockElements = new ArrayList();
            _htmlBlockElements.Add("blockquote");
            _htmlBlockElements.Add("body");
            _htmlBlockElements.Add("caption");
            _htmlBlockElements.Add("center");
            _htmlBlockElements.Add("cite");
            _htmlBlockElements.Add("dd");
            _htmlBlockElements.Add("dir");
            _htmlBlockElements.Add("div");
            _htmlBlockElements.Add("dl");
            _htmlBlockElements.Add("dt");
            _htmlBlockElements.Add("form");
            _htmlBlockElements.Add("h1");
            _htmlBlockElements.Add("h2");
            _htmlBlockElements.Add("h3");
            _htmlBlockElements.Add("h4");
            _htmlBlockElements.Add("h5");
            _htmlBlockElements.Add("h6");
            _htmlBlockElements.Add("html");
            _htmlBlockElements.Add("li");
            _htmlBlockElements.Add("menu");
            _htmlBlockElements.Add("ol");
            _htmlBlockElements.Add("p");
            _htmlBlockElements.Add("pre");
            _htmlBlockElements.Add("table");
            _htmlBlockElements.Add("tbody");
            _htmlBlockElements.Add("td");
            _htmlBlockElements.Add("textarea");
            _htmlBlockElements.Add("tfoot");
            _htmlBlockElements.Add("th");
            _htmlBlockElements.Add("thead");
            _htmlBlockElements.Add("tr");
            _htmlBlockElements.Add("tt");
            _htmlBlockElements.Add("ul");
        }

        private static void InitializeElementsClosingOnNewElementStart()
        {
            _htmlElementsClosingColgroup = new ArrayList();
            _htmlElementsClosingColgroup.Add("colgroup");
            _htmlElementsClosingColgroup.Add("tr");
            _htmlElementsClosingColgroup.Add("thead");
            _htmlElementsClosingColgroup.Add("tfoot");
            _htmlElementsClosingColgroup.Add("tbody");
            _htmlElementsClosingDD = new ArrayList();
            _htmlElementsClosingDD.Add("dd");
            _htmlElementsClosingDD.Add("dt");
            _htmlElementsClosingDT = new ArrayList();
            _htmlElementsClosingDD.Add("dd");
            _htmlElementsClosingDD.Add("dt");
            _htmlElementsClosingLI = new ArrayList();
            _htmlElementsClosingLI.Add("li");
            _htmlElementsClosingTbody = new ArrayList();
            _htmlElementsClosingTbody.Add("tbody");
            _htmlElementsClosingTbody.Add("thead");
            _htmlElementsClosingTbody.Add("tfoot");
            _htmlElementsClosingTR = new ArrayList();
            _htmlElementsClosingTR.Add("thead");
            _htmlElementsClosingTR.Add("tfoot");
            _htmlElementsClosingTR.Add("tbody");
            _htmlElementsClosingTR.Add("tr");
            _htmlElementsClosingTD = new ArrayList();
            _htmlElementsClosingTD.Add("td");
            _htmlElementsClosingTD.Add("th");
            _htmlElementsClosingTD.Add("tr");
            _htmlElementsClosingTD.Add("tbody");
            _htmlElementsClosingTD.Add("tfoot");
            _htmlElementsClosingTD.Add("thead");
            _htmlElementsClosingTH = new ArrayList();
            _htmlElementsClosingTH.Add("td");
            _htmlElementsClosingTH.Add("th");
            _htmlElementsClosingTH.Add("tr");
            _htmlElementsClosingTH.Add("tbody");
            _htmlElementsClosingTH.Add("tfoot");
            _htmlElementsClosingTH.Add("thead");
            _htmlElementsClosingThead = new ArrayList();
            _htmlElementsClosingThead.Add("tbody");
            _htmlElementsClosingThead.Add("tfoot");
            _htmlElementsClosingTfoot = new ArrayList();
            _htmlElementsClosingTfoot.Add("tbody");
            _htmlElementsClosingTfoot.Add("thead");
        }

        private static void InitializeElementsClosingOnParentElementEnd()
        {
            _htmlElementsClosingOnParentElementEnd = new ArrayList();
            _htmlElementsClosingOnParentElementEnd.Add("body");
            _htmlElementsClosingOnParentElementEnd.Add("colgroup");
            _htmlElementsClosingOnParentElementEnd.Add("dd");
            _htmlElementsClosingOnParentElementEnd.Add("dt");
            _htmlElementsClosingOnParentElementEnd.Add("head");
            _htmlElementsClosingOnParentElementEnd.Add("html");
            _htmlElementsClosingOnParentElementEnd.Add("li");
            _htmlElementsClosingOnParentElementEnd.Add("p");
            _htmlElementsClosingOnParentElementEnd.Add("tbody");
            _htmlElementsClosingOnParentElementEnd.Add("td");
            _htmlElementsClosingOnParentElementEnd.Add("tfoot");
            _htmlElementsClosingOnParentElementEnd.Add("thead");
            _htmlElementsClosingOnParentElementEnd.Add("th");
            _htmlElementsClosingOnParentElementEnd.Add("tr");
        }

        private static void InitializeEmptyElements()
        {
            _htmlEmptyElements = new ArrayList();
            _htmlEmptyElements.Add("area");
            _htmlEmptyElements.Add("base");
            _htmlEmptyElements.Add("basefont");
            _htmlEmptyElements.Add("br");
            _htmlEmptyElements.Add("col");
            _htmlEmptyElements.Add("frame");
            _htmlEmptyElements.Add("hr");
            _htmlEmptyElements.Add("img");
            _htmlEmptyElements.Add("input");
            _htmlEmptyElements.Add("isindex");
            _htmlEmptyElements.Add("link");
            _htmlEmptyElements.Add("meta");
            _htmlEmptyElements.Add("param");
        }

        private static void InitializeHtmlCharacterEntities()
        {
            _htmlCharacterEntities = new Hashtable();
            _htmlCharacterEntities["Aacute"] = '\x00c1';
            _htmlCharacterEntities["aacute"] = '\x00e1';
            _htmlCharacterEntities["Acirc"] = '\x00c2';
            _htmlCharacterEntities["acirc"] = '\x00e2';
            _htmlCharacterEntities["acute"] = '\x00b4';
            _htmlCharacterEntities["AElig"] = '\x00c6';
            _htmlCharacterEntities["aelig"] = '\x00e6';
            _htmlCharacterEntities["Agrave"] = '\x00c0';
            _htmlCharacterEntities["agrave"] = '\x00e0';
            _htmlCharacterEntities["alefsym"] = 'ℵ';
            _htmlCharacterEntities["Alpha"] = 'Α';
            _htmlCharacterEntities["alpha"] = 'α';
            _htmlCharacterEntities["amp"] = '&';
            _htmlCharacterEntities["and"] = '∧';
            _htmlCharacterEntities["ang"] = '∠';
            _htmlCharacterEntities["Aring"] = '\x00c5';
            _htmlCharacterEntities["aring"] = '\x00e5';
            _htmlCharacterEntities["asymp"] = '≈';
            _htmlCharacterEntities["Atilde"] = '\x00c3';
            _htmlCharacterEntities["atilde"] = '\x00e3';
            _htmlCharacterEntities["Auml"] = '\x00c4';
            _htmlCharacterEntities["auml"] = '\x00e4';
            _htmlCharacterEntities["bdquo"] = '„';
            _htmlCharacterEntities["Beta"] = 'Β';
            _htmlCharacterEntities["beta"] = 'β';
            _htmlCharacterEntities["brvbar"] = '\x00a6';
            _htmlCharacterEntities["bull"] = '•';
            _htmlCharacterEntities["cap"] = '∩';
            _htmlCharacterEntities["Ccedil"] = '\x00c7';
            _htmlCharacterEntities["ccedil"] = '\x00e7';
            _htmlCharacterEntities["cent"] = '\x00a2';
            _htmlCharacterEntities["Chi"] = 'Χ';
            _htmlCharacterEntities["chi"] = 'χ';
            _htmlCharacterEntities["circ"] = 'ˆ';
            _htmlCharacterEntities["clubs"] = '♣';
            _htmlCharacterEntities["cong"] = '≅';
            _htmlCharacterEntities["copy"] = '\x00a9';
            _htmlCharacterEntities["crarr"] = '↵';
            _htmlCharacterEntities["cup"] = '∪';
            _htmlCharacterEntities["curren"] = '\x00a4';
            _htmlCharacterEntities["dagger"] = '†';
            _htmlCharacterEntities["Dagger"] = '‡';
            _htmlCharacterEntities["darr"] = '↓';
            _htmlCharacterEntities["dArr"] = '⇓';
            _htmlCharacterEntities["deg"] = '\x00b0';
            _htmlCharacterEntities["Delta"] = 'Δ';
            _htmlCharacterEntities["delta"] = 'δ';
            _htmlCharacterEntities["diams"] = '♦';
            _htmlCharacterEntities["divide"] = '\x00f7';
            _htmlCharacterEntities["Eacute"] = '\x00c9';
            _htmlCharacterEntities["eacute"] = '\x00e9';
            _htmlCharacterEntities["Ecirc"] = '\x00ca';
            _htmlCharacterEntities["ecirc"] = '\x00ea';
            _htmlCharacterEntities["Egrave"] = '\x00c8';
            _htmlCharacterEntities["egrave"] = '\x00e8';
            _htmlCharacterEntities["empty"] = '∅';
            _htmlCharacterEntities["emsp"] = ' ';
            _htmlCharacterEntities["ensp"] = ' ';
            _htmlCharacterEntities["Epsilon"] = 'Ε';
            _htmlCharacterEntities["epsilon"] = 'ε';
            _htmlCharacterEntities["equiv"] = '≡';
            _htmlCharacterEntities["Eta"] = 'Η';
            _htmlCharacterEntities["eta"] = 'η';
            _htmlCharacterEntities["ETH"] = '\x00d0';
            _htmlCharacterEntities["eth"] = '\x00f0';
            _htmlCharacterEntities["Euml"] = '\x00cb';
            _htmlCharacterEntities["euml"] = '\x00eb';
            _htmlCharacterEntities["euro"] = '€';
            _htmlCharacterEntities["exist"] = '∃';
            _htmlCharacterEntities["fnof"] = 'ƒ';
            _htmlCharacterEntities["forall"] = '∀';
            _htmlCharacterEntities["frac12"] = '\x00bd';
            _htmlCharacterEntities["frac14"] = '\x00bc';
            _htmlCharacterEntities["frac34"] = '\x00be';
            _htmlCharacterEntities["frasl"] = '⁄';
            _htmlCharacterEntities["Gamma"] = 'Γ';
            _htmlCharacterEntities["gamma"] = 'γ';
            _htmlCharacterEntities["ge"] = '≥';
            _htmlCharacterEntities["gt"] = '>';
            _htmlCharacterEntities["harr"] = '↔';
            _htmlCharacterEntities["hArr"] = '⇔';
            _htmlCharacterEntities["hearts"] = '♥';
            _htmlCharacterEntities["hellip"] = '…';
            _htmlCharacterEntities["Iacute"] = '\x00cd';
            _htmlCharacterEntities["iacute"] = '\x00ed';
            _htmlCharacterEntities["Icirc"] = '\x00ce';
            _htmlCharacterEntities["icirc"] = '\x00ee';
            _htmlCharacterEntities["iexcl"] = '\x00a1';
            _htmlCharacterEntities["Igrave"] = '\x00cc';
            _htmlCharacterEntities["igrave"] = '\x00ec';
            _htmlCharacterEntities["image"] = 'ℑ';
            _htmlCharacterEntities["infin"] = '∞';
            _htmlCharacterEntities["int"] = '∫';
            _htmlCharacterEntities["Iota"] = 'Ι';
            _htmlCharacterEntities["iota"] = 'ι';
            _htmlCharacterEntities["iquest"] = '\x00bf';
            _htmlCharacterEntities["isin"] = '∈';
            _htmlCharacterEntities["Iuml"] = '\x00cf';
            _htmlCharacterEntities["iuml"] = '\x00ef';
            _htmlCharacterEntities["Kappa"] = 'Κ';
            _htmlCharacterEntities["kappa"] = 'κ';
            _htmlCharacterEntities["Lambda"] = 'Λ';
            _htmlCharacterEntities["lambda"] = 'λ';
            _htmlCharacterEntities["lang"] = '〈';
            _htmlCharacterEntities["laquo"] = '\x00ab';
            _htmlCharacterEntities["larr"] = '←';
            _htmlCharacterEntities["lArr"] = '⇐';
            _htmlCharacterEntities["lceil"] = '⌈';
            _htmlCharacterEntities["ldquo"] = '“';
            _htmlCharacterEntities["le"] = '≤';
            _htmlCharacterEntities["lfloor"] = '⌊';
            _htmlCharacterEntities["lowast"] = '∗';
            _htmlCharacterEntities["loz"] = '◊';
            _htmlCharacterEntities["lrm"] = '‎';
            _htmlCharacterEntities["lsaquo"] = '‹';
            _htmlCharacterEntities["lsquo"] = '‘';
            _htmlCharacterEntities["lt"] = '<';
            _htmlCharacterEntities["macr"] = '\x00af';
            _htmlCharacterEntities["mdash"] = '—';
            _htmlCharacterEntities["micro"] = '\x00b5';
            _htmlCharacterEntities["middot"] = '\x00b7';
            _htmlCharacterEntities["minus"] = '−';
            _htmlCharacterEntities["Mu"] = 'Μ';
            _htmlCharacterEntities["mu"] = 'μ';
            _htmlCharacterEntities["nabla"] = '∇';
            _htmlCharacterEntities["nbsp"] = '\x00a0';
            _htmlCharacterEntities["ndash"] = '–';
            _htmlCharacterEntities["ne"] = '≠';
            _htmlCharacterEntities["ni"] = '∋';
            _htmlCharacterEntities["not"] = '\x00ac';
            _htmlCharacterEntities["notin"] = '∉';
            _htmlCharacterEntities["nsub"] = '⊄';
            _htmlCharacterEntities["Ntilde"] = '\x00d1';
            _htmlCharacterEntities["ntilde"] = '\x00f1';
            _htmlCharacterEntities["Nu"] = 'Ν';
            _htmlCharacterEntities["nu"] = 'ν';
            _htmlCharacterEntities["Oacute"] = '\x00d3';
            _htmlCharacterEntities["ocirc"] = '\x00f4';
            _htmlCharacterEntities["OElig"] = 'Œ';
            _htmlCharacterEntities["oelig"] = 'œ';
            _htmlCharacterEntities["Ograve"] = '\x00d2';
            _htmlCharacterEntities["ograve"] = '\x00f2';
            _htmlCharacterEntities["oline"] = '‾';
            _htmlCharacterEntities["Omega"] = 'Ω';
            _htmlCharacterEntities["omega"] = 'ω';
            _htmlCharacterEntities["Omicron"] = 'Ο';
            _htmlCharacterEntities["omicron"] = 'ο';
            _htmlCharacterEntities["oplus"] = '⊕';
            _htmlCharacterEntities["or"] = '∨';
            _htmlCharacterEntities["ordf"] = '\x00aa';
            _htmlCharacterEntities["ordm"] = '\x00ba';
            _htmlCharacterEntities["Oslash"] = '\x00d8';
            _htmlCharacterEntities["oslash"] = '\x00f8';
            _htmlCharacterEntities["Otilde"] = '\x00d5';
            _htmlCharacterEntities["otilde"] = '\x00f5';
            _htmlCharacterEntities["otimes"] = '⊗';
            _htmlCharacterEntities["Ouml"] = '\x00d6';
            _htmlCharacterEntities["ouml"] = '\x00f6';
            _htmlCharacterEntities["para"] = '\x00b6';
            _htmlCharacterEntities["part"] = '∂';
            _htmlCharacterEntities["permil"] = '‰';
            _htmlCharacterEntities["perp"] = '⊥';
            _htmlCharacterEntities["Phi"] = 'Φ';
            _htmlCharacterEntities["phi"] = 'φ';
            _htmlCharacterEntities["pi"] = 'π';
            _htmlCharacterEntities["piv"] = 'ϖ';
            _htmlCharacterEntities["plusmn"] = '\x00b1';
            _htmlCharacterEntities["pound"] = '\x00a3';
            _htmlCharacterEntities["prime"] = '′';
            _htmlCharacterEntities["Prime"] = '″';
            _htmlCharacterEntities["prod"] = '∏';
            _htmlCharacterEntities["prop"] = '∝';
            _htmlCharacterEntities["Psi"] = 'Ψ';
            _htmlCharacterEntities["psi"] = 'ψ';
            _htmlCharacterEntities["quot"] = '"';
            _htmlCharacterEntities["radic"] = '√';
            _htmlCharacterEntities["rang"] = '〉';
            _htmlCharacterEntities["raquo"] = '\x00bb';
            _htmlCharacterEntities["rarr"] = '→';
            _htmlCharacterEntities["rArr"] = '⇒';
            _htmlCharacterEntities["rceil"] = '⌉';
            _htmlCharacterEntities["rdquo"] = '”';
            _htmlCharacterEntities["real"] = 'ℜ';
            _htmlCharacterEntities["reg"] = '\x00ae';
            _htmlCharacterEntities["rfloor"] = '⌋';
            _htmlCharacterEntities["Rho"] = 'Ρ';
            _htmlCharacterEntities["rho"] = 'ρ';
            _htmlCharacterEntities["rlm"] = '‏';
            _htmlCharacterEntities["rsaquo"] = '›';
            _htmlCharacterEntities["rsquo"] = '’';
            _htmlCharacterEntities["sbquo"] = '‚';
            _htmlCharacterEntities["Scaron"] = 'Š';
            _htmlCharacterEntities["scaron"] = 'š';
            _htmlCharacterEntities["sdot"] = '⋅';
            _htmlCharacterEntities["sect"] = '\x00a7';
            _htmlCharacterEntities["shy"] = '\x00ad';
            _htmlCharacterEntities["Sigma"] = 'Σ';
            _htmlCharacterEntities["sigma"] = 'σ';
            _htmlCharacterEntities["sigmaf"] = 'ς';
            _htmlCharacterEntities["sim"] = '∼';
            _htmlCharacterEntities["spades"] = '♠';
            _htmlCharacterEntities["sub"] = '⊂';
            _htmlCharacterEntities["sube"] = '⊆';
            _htmlCharacterEntities["sum"] = '∑';
            _htmlCharacterEntities["sup"] = '⊃';
            _htmlCharacterEntities["sup1"] = '\x00b9';
            _htmlCharacterEntities["sup2"] = '\x00b2';
            _htmlCharacterEntities["sup3"] = '\x00b3';
            _htmlCharacterEntities["supe"] = '⊇';
            _htmlCharacterEntities["szlig"] = '\x00df';
            _htmlCharacterEntities["Tau"] = 'Τ';
            _htmlCharacterEntities["tau"] = 'τ';
            _htmlCharacterEntities["there4"] = '∴';
            _htmlCharacterEntities["Theta"] = 'Θ';
            _htmlCharacterEntities["theta"] = 'θ';
            _htmlCharacterEntities["thetasym"] = 'ϑ';
            _htmlCharacterEntities["thinsp"] = ' ';
            _htmlCharacterEntities["THORN"] = '\x00de';
            _htmlCharacterEntities["thorn"] = '\x00fe';
            _htmlCharacterEntities["tilde"] = '˜';
            _htmlCharacterEntities["times"] = '\x00d7';
            _htmlCharacterEntities["trade"] = '™';
            _htmlCharacterEntities["Uacute"] = '\x00da';
            _htmlCharacterEntities["uacute"] = '\x00fa';
            _htmlCharacterEntities["uarr"] = '↑';
            _htmlCharacterEntities["uArr"] = '⇑';
            _htmlCharacterEntities["Ucirc"] = '\x00db';
            _htmlCharacterEntities["ucirc"] = '\x00fb';
            _htmlCharacterEntities["Ugrave"] = '\x00d9';
            _htmlCharacterEntities["ugrave"] = '\x00f9';
            _htmlCharacterEntities["uml"] = '\x00a8';
            _htmlCharacterEntities["upsih"] = 'ϒ';
            _htmlCharacterEntities["Upsilon"] = 'Υ';
            _htmlCharacterEntities["upsilon"] = 'υ';
            _htmlCharacterEntities["Uuml"] = '\x00dc';
            _htmlCharacterEntities["uuml"] = '\x00fc';
            _htmlCharacterEntities["weierp"] = '℘';
            _htmlCharacterEntities["Xi"] = 'Ξ';
            _htmlCharacterEntities["xi"] = 'ξ';
            _htmlCharacterEntities["Yacute"] = '\x00dd';
            _htmlCharacterEntities["yacute"] = '\x00fd';
            _htmlCharacterEntities["yen"] = '\x00a5';
            _htmlCharacterEntities["Yuml"] = 'Ÿ';
            _htmlCharacterEntities["yuml"] = '\x00ff';
            _htmlCharacterEntities["Zeta"] = 'Ζ';
            _htmlCharacterEntities["zeta"] = 'ζ';
            _htmlCharacterEntities["zwj"] = '‍';
            _htmlCharacterEntities["zwnj"] = '‌';
        }

        private static void InitializeInlineElements()
        {
            _htmlInlineElements = new ArrayList();
            _htmlInlineElements.Add("a");
            _htmlInlineElements.Add("abbr");
            _htmlInlineElements.Add("acronym");
            _htmlInlineElements.Add("address");
            _htmlInlineElements.Add("b");
            _htmlInlineElements.Add("bdo");
            _htmlInlineElements.Add("big");
            _htmlInlineElements.Add("button");
            _htmlInlineElements.Add("code");
            _htmlInlineElements.Add("del");
            _htmlInlineElements.Add("dfn");
            _htmlInlineElements.Add("em");
            _htmlInlineElements.Add("font");
            _htmlInlineElements.Add("i");
            _htmlInlineElements.Add("ins");
            _htmlInlineElements.Add("kbd");
            _htmlInlineElements.Add("label");
            _htmlInlineElements.Add("legend");
            _htmlInlineElements.Add("q");
            _htmlInlineElements.Add("s");
            _htmlInlineElements.Add("samp");
            _htmlInlineElements.Add("small");
            _htmlInlineElements.Add("span");
            _htmlInlineElements.Add("strike");
            _htmlInlineElements.Add("strong");
            _htmlInlineElements.Add("sub");
            _htmlInlineElements.Add("sup");
            _htmlInlineElements.Add("u");
            _htmlInlineElements.Add("var");
        }

        private static void InitializeOtherOpenableElements()
        {
            _htmlOtherOpenableElements = new ArrayList();
            _htmlOtherOpenableElements.Add("applet");
            _htmlOtherOpenableElements.Add("base");
            _htmlOtherOpenableElements.Add("basefont");
            _htmlOtherOpenableElements.Add("colgroup");
            _htmlOtherOpenableElements.Add("fieldset");
            _htmlOtherOpenableElements.Add("frameset");
            _htmlOtherOpenableElements.Add("head");
            _htmlOtherOpenableElements.Add("iframe");
            _htmlOtherOpenableElements.Add("map");
            _htmlOtherOpenableElements.Add("noframes");
            _htmlOtherOpenableElements.Add("noscript");
            _htmlOtherOpenableElements.Add("object");
            _htmlOtherOpenableElements.Add("optgroup");
            _htmlOtherOpenableElements.Add("option");
            _htmlOtherOpenableElements.Add("script");
            _htmlOtherOpenableElements.Add("select");
            _htmlOtherOpenableElements.Add("style");
            _htmlOtherOpenableElements.Add("title");
        }

        internal static bool IsBlockElement(string xmlElementName)
        {
            return _htmlBlockElements.Contains(xmlElementName);
        }

        internal static bool IsEmptyElement(string xmlElementName)
        {
            return _htmlEmptyElements.Contains(xmlElementName.ToLower());
        }

        internal static bool IsEntity(string entityName)
        {
            return _htmlCharacterEntities.Contains(entityName);
        }

        internal static bool IsInlineElement(string xmlElementName)
        {
            return _htmlInlineElements.Contains(xmlElementName);
        }

        internal static bool IsKnownOpenableElement(string xmlElementName)
        {
            return _htmlOtherOpenableElements.Contains(xmlElementName);
        }
    }
}

