﻿namespace HTMLConverter
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Runtime.InteropServices;
    using System.Windows;
    using System.Windows.Documents;
    using System.Xml;

    public static class HtmlToXamlConverter
    {
        private static string _xamlNamespace = "http://schemas.microsoft.com/winfx/2006/xaml/presentation";
        private static XmlElement InlineFragmentParentElement;
        public const string Xaml_Background = "Background";
        public const string Xaml_BorderBrush = "BorderBrush";
        public const string Xaml_BorderThickness = "BorderThickness";
        public const string Xaml_Brushes_Black = "Black";
        public const string Xaml_FlowDocument = "FlowDocument";
        public const string Xaml_FontFamily = "FontFamily";
        public const string Xaml_FontSize = "FontSize";
        public const string Xaml_FontSize_Large = "18pt";
        public const string Xaml_FontSize_Medium = "16pt";
        public const string Xaml_FontSize_Small = "12pt";
        public const string Xaml_FontSize_XLarge = "20pt";
        public const string Xaml_FontSize_XSmall = "10pt";
        public const string Xaml_FontSize_XXLarge = "22pt";
        public const string Xaml_FontSize_XXSmall = "8pt";
        public const string Xaml_FontStyle = "FontStyle";
        public const string Xaml_FontWeight = "FontWeight";
        public const string Xaml_FontWeight_Bold = "Bold";
        public const string Xaml_Foreground = "Foreground";
        public const string Xaml_Hyperlink = "Hyperlink";
        public const string Xaml_Hyperlink_NavigateUri = "NavigateUri";
        public const string Xaml_Hyperlink_TargetName = "TargetName";
        public const string Xaml_LineBreak = "LineBreak";
        public const string Xaml_List = "List";
        public const string Xaml_List_MarkerStyle = "MarkerStyle";
        public const string Xaml_List_MarkerStyle_Box = "Box";
        public const string Xaml_List_MarkerStyle_Circle = "Circle";
        public const string Xaml_List_MarkerStyle_Decimal = "Decimal";
        public const string Xaml_List_MarkerStyle_Disc = "Disc";
        public const string Xaml_List_MarkerStyle_LowerLatin = "LowerLatin";
        public const string Xaml_List_MarkerStyle_LowerRoman = "LowerRoman";
        public const string Xaml_List_MarkerStyle_None = "None";
        public const string Xaml_List_MarkerStyle_Square = "Square";
        public const string Xaml_List_MarkerStyle_UpperLatin = "UpperLatin";
        public const string Xaml_List_MarkerStyle_UpperRoman = "UpperRoman";
        public const string Xaml_ListItem = "ListItem";
        public const string Xaml_Margin = "Margin";
        public const string Xaml_Padding = "Padding";
        public const string Xaml_Paragraph = "Paragraph";
        public const string Xaml_Run = "Run";
        public const string Xaml_Section = "Section";
        public const string Xaml_Span = "Span";
        public const string Xaml_Table = "Table";
        public const string Xaml_TableCell = "TableCell";
        public const string Xaml_TableCell_BorderBrush = "BorderBrush";
        public const string Xaml_TableCell_BorderThickness = "BorderThickness";
        public const string Xaml_TableCell_ColumnSpan = "ColumnSpan";
        public const string Xaml_TableCell_RowSpan = "RowSpan";
        public const string Xaml_TableColumn = "TableColumn";
        public const string Xaml_TableRow = "TableRow";
        public const string Xaml_TableRowGroup = "TableRowGroup";
        public const string Xaml_TextAlignment = "TextAlignment";
        public const string Xaml_TextDecorations = "TextDecorations";
        public const string Xaml_TextDecorations_Underline = "Underline";
        public const string Xaml_TextIndent = "TextIndent";
        public const string Xaml_Width = "Width";

        private static XmlNode AddBlock(XmlElement xamlParentElement, XmlNode htmlNode, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            if (htmlNode is XmlComment)
            {
                DefineInlineFragmentParent((XmlComment) htmlNode, null);
                return htmlNode;
            }
            if (htmlNode is XmlText)
            {
                htmlNode = AddImplicitParagraph(xamlParentElement, htmlNode, inheritedProperties, stylesheet, sourceContext);
                return htmlNode;
            }
            if (htmlNode is XmlElement)
            {
                XmlElement item = (XmlElement) htmlNode;
                string localName = item.LocalName;
                if (item.NamespaceURI != "http://www.w3.org/1999/xhtml")
                {
                    return item;
                }
                sourceContext.Add(item);
                switch (localName.ToLower())
                {
                    case "html":
                    case "body":
                    case "div":
                    case "form":
                    case "pre":
                    case "blockquote":
                    case "caption":
                    case "center":
                    case "cite":
                        AddSection(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
                        break;

                    case "p":
                    case "h1":
                    case "h2":
                    case "h3":
                    case "h4":
                    case "h5":
                    case "h6":
                    case "nsrtitle":
                    case "textarea":
                    case "dd":
                    case "dl":
                    case "dt":
                    case "tt":
                        AddParagraph(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
                        break;

                    case "ol":
                    case "ul":
                    case "dir":
                    case "menu":
                        AddList(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
                        break;

                    case "li":
                        htmlNode = AddOrphanListItems(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
                        break;

                    case "img":
                        AddImage(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
                        break;

                    case "table":
                        AddTable(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
                        break;

                    case "style":
                    case "meta":
                    case "head":
                    case "title":
                    case "script":
                        break;

                    default:
                        htmlNode = AddImplicitParagraph(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
                        break;
                }
                Debug.Assert((sourceContext.Count > 0) && (sourceContext[sourceContext.Count - 1] == item));
                sourceContext.RemoveAt(sourceContext.Count - 1);
            }
            return htmlNode;
        }

        private static void AddBreak(XmlElement xamlParentElement, string htmlElementName)
        {
            XmlElement newChild = xamlParentElement.OwnerDocument.CreateElement(null, "LineBreak", _xamlNamespace);
            xamlParentElement.AppendChild(newChild);
            if (htmlElementName == "hr")
            {
                XmlText text = xamlParentElement.OwnerDocument.CreateTextNode("----------------------");
                xamlParentElement.AppendChild(text);
                newChild = xamlParentElement.OwnerDocument.CreateElement(null, "LineBreak", _xamlNamespace);
                xamlParentElement.AppendChild(newChild);
            }
        }

        private static void AddColumnInformation(XmlElement htmlTableElement, XmlElement xamlTableElement, ArrayList columnStartsAllRows, Hashtable currentProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            if (columnStartsAllRows != null)
            {
                for (int i = 0; i < (columnStartsAllRows.Count - 1); i++)
                {
                    XmlElement newChild = xamlTableElement.OwnerDocument.CreateElement(null, "TableColumn", _xamlNamespace);
                    newChild.SetAttribute("Width", (((double) columnStartsAllRows[i + 1]) - ((double) columnStartsAllRows[i])).ToString());
                    xamlTableElement.AppendChild(newChild);
                }
            }
            else
            {
                for (XmlNode node = htmlTableElement.FirstChild; node != null; node = node.NextSibling)
                {
                    if (node.LocalName.ToLower() == "colgroup")
                    {
                        AddTableColumnGroup(xamlTableElement, (XmlElement) node, currentProperties, stylesheet, sourceContext);
                    }
                    else if (node.LocalName.ToLower() == "col")
                    {
                        AddTableColumn(xamlTableElement, (XmlElement) node, currentProperties, stylesheet, sourceContext);
                    }
                    else if (node is XmlElement)
                    {
                        break;
                    }
                }
            }
        }

        private static void AddDataToTableCell(XmlElement xamlTableCellElement, XmlNode htmlDataStartNode, Hashtable currentProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Debug.Assert(xamlTableCellElement.LocalName == "TableCell");
            Debug.Assert(currentProperties != null);
            for (XmlNode node = htmlDataStartNode; node != null; node = (node != null) ? node.NextSibling : null)
            {
                node = AddBlock(xamlTableCellElement, node, currentProperties, stylesheet, sourceContext);
            }
        }

        private static void AddHyperlink(XmlElement xamlParentElement, XmlElement htmlElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            string attribute = GetAttribute(htmlElement, "href");
            if (attribute == null)
            {
                AddSpanOrRun(xamlParentElement, htmlElement, inheritedProperties, stylesheet, sourceContext);
            }
            else
            {
                Hashtable hashtable;
                Hashtable hashtable2 = GetElementProperties(htmlElement, inheritedProperties, out hashtable, stylesheet, sourceContext);
                XmlElement xamlElement = xamlParentElement.OwnerDocument.CreateElement(null, "Hyperlink", _xamlNamespace);
                ApplyLocalProperties(xamlElement, hashtable, false);
                string[] strArray = attribute.Split(new char[] { '#' });
                if ((strArray.Length > 0) && (strArray[0].Trim().Length > 0))
                {
                    xamlElement.SetAttribute("NavigateUri", strArray[0].Trim());
                }
                if ((strArray.Length == 2) && (strArray[1].Trim().Length > 0))
                {
                    xamlElement.SetAttribute("TargetName", strArray[1].Trim());
                }
                for (XmlNode node = htmlElement.FirstChild; node != null; node = node.NextSibling)
                {
                    AddInline(xamlElement, node, hashtable2, stylesheet, sourceContext);
                }
                xamlParentElement.AppendChild(xamlElement);
            }
        }

        private static void AddImage(XmlElement xamlParentElement, XmlElement htmlElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
        }

        private static XmlNode AddImplicitParagraph(XmlElement xamlParentElement, XmlNode htmlNode, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            XmlElement xamlElement = xamlParentElement.OwnerDocument.CreateElement(null, "Paragraph", _xamlNamespace);
            XmlNode node = null;
            while (htmlNode != null)
            {
                if (htmlNode is XmlComment)
                {
                    DefineInlineFragmentParent((XmlComment) htmlNode, null);
                }
                else if (htmlNode is XmlText)
                {
                    if (htmlNode.Value.Trim().Length > 0)
                    {
                        AddTextRun(xamlElement, htmlNode.Value);
                    }
                }
                else if (htmlNode is XmlElement)
                {
                    if (HtmlSchema.IsBlockElement(((XmlElement) htmlNode).LocalName.ToLower()))
                    {
                        break;
                    }
                    AddInline(xamlElement, (XmlElement) htmlNode, inheritedProperties, stylesheet, sourceContext);
                }
                node = htmlNode;
                htmlNode = htmlNode.NextSibling;
            }
            if (xamlElement.FirstChild != null)
            {
                xamlParentElement.AppendChild(xamlElement);
            }
            return node;
        }

        private static void AddInline(XmlElement xamlParentElement, XmlNode htmlNode, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            if (htmlNode is XmlComment)
            {
                DefineInlineFragmentParent((XmlComment) htmlNode, xamlParentElement);
                return;
            }
            if (htmlNode is XmlText)
            {
                AddTextRun(xamlParentElement, htmlNode.Value);
                return;
            }
            if (!(htmlNode is XmlElement))
            {
                return;
            }
            XmlElement item = (XmlElement) htmlNode;
            if (item.NamespaceURI != "http://www.w3.org/1999/xhtml")
            {
                return;
            }
            string htmlElementName = item.LocalName.ToLower();
            sourceContext.Add(item);
            string str2 = htmlElementName;
            if (str2 != null)
            {
                if (!(str2 == "a"))
                {
                    if (str2 == "img")
                    {
                        AddImage(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
                        goto Label_011F;
                    }
                    if ((str2 == "br") || (str2 == "hr"))
                    {
                        AddBreak(xamlParentElement, htmlElementName);
                        goto Label_011F;
                    }
                }
                else
                {
                    AddHyperlink(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
                    goto Label_011F;
                }
            }
            if (HtmlSchema.IsInlineElement(htmlElementName) || HtmlSchema.IsBlockElement(htmlElementName))
            {
                AddSpanOrRun(xamlParentElement, item, inheritedProperties, stylesheet, sourceContext);
            }
        Label_011F:
            Debug.Assert((sourceContext.Count > 0) && (sourceContext[sourceContext.Count - 1] == item));
            sourceContext.RemoveAt(sourceContext.Count - 1);
        }

        private static void AddList(XmlElement xamlParentElement, XmlElement htmlListElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Hashtable hashtable;
            string str = htmlListElement.LocalName.ToLower();
            Hashtable hashtable2 = GetElementProperties(htmlListElement, inheritedProperties, out hashtable, stylesheet, sourceContext);
            XmlElement xamlElement = xamlParentElement.OwnerDocument.CreateElement(null, "List", _xamlNamespace);
            if (str == "ol")
            {
                xamlElement.SetAttribute("MarkerStyle", "Decimal");
            }
            else
            {
                xamlElement.SetAttribute("MarkerStyle", "Disc");
            }
            ApplyLocalProperties(xamlElement, hashtable, true);
            for (XmlNode node = htmlListElement.FirstChild; node != null; node = node.NextSibling)
            {
                if ((node is XmlElement) && (node.LocalName.ToLower() == "li"))
                {
                    sourceContext.Add((XmlElement) node);
                    AddListItem(xamlElement, (XmlElement) node, hashtable2, stylesheet, sourceContext);
                    Debug.Assert((sourceContext.Count > 0) && (sourceContext[sourceContext.Count - 1] == node));
                    sourceContext.RemoveAt(sourceContext.Count - 1);
                }
            }
            if (xamlElement.HasChildNodes)
            {
                xamlParentElement.AppendChild(xamlElement);
            }
        }

        private static void AddListItem(XmlElement xamlListElement, XmlElement htmlLIElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Hashtable hashtable;
            Debug.Assert(xamlListElement != null);
            Debug.Assert(xamlListElement.LocalName == "List");
            Debug.Assert(htmlLIElement != null);
            Debug.Assert(htmlLIElement.LocalName.ToLower() == "li");
            Debug.Assert(inheritedProperties != null);
            Hashtable hashtable2 = GetElementProperties(htmlLIElement, inheritedProperties, out hashtable, stylesheet, sourceContext);
            XmlElement xamlParentElement = xamlListElement.OwnerDocument.CreateElement(null, "ListItem", _xamlNamespace);
            for (XmlNode node = htmlLIElement.FirstChild; node != null; node = (node != null) ? node.NextSibling : null)
            {
                node = AddBlock(xamlParentElement, node, hashtable2, stylesheet, sourceContext);
            }
            xamlListElement.AppendChild(xamlParentElement);
        }

        private static XmlElement AddOrphanListItems(XmlElement xamlParentElement, XmlElement htmlLIElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            XmlElement element2;
            Debug.Assert(htmlLIElement.LocalName.ToLower() == "li");
            XmlElement element = null;
            XmlNode lastChild = xamlParentElement.LastChild;
            if ((lastChild != null) && (lastChild.LocalName == "List"))
            {
                element2 = (XmlElement) lastChild;
            }
            else
            {
                element2 = xamlParentElement.OwnerDocument.CreateElement(null, "List", _xamlNamespace);
                xamlParentElement.AppendChild(element2);
            }
            XmlNode nextSibling = htmlLIElement;
            for (string str = (nextSibling == null) ? null : nextSibling.LocalName.ToLower(); (nextSibling != null) && (str == "li"); str = (nextSibling == null) ? null : nextSibling.LocalName.ToLower())
            {
                AddListItem(element2, (XmlElement) nextSibling, inheritedProperties, stylesheet, sourceContext);
                element = (XmlElement) nextSibling;
                nextSibling = nextSibling.NextSibling;
            }
            return element;
        }

        private static void AddParagraph(XmlElement xamlParentElement, XmlElement htmlElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Hashtable hashtable;
            Hashtable hashtable2 = GetElementProperties(htmlElement, inheritedProperties, out hashtable, stylesheet, sourceContext);
            XmlElement xamlElement = xamlParentElement.OwnerDocument.CreateElement(null, "Paragraph", _xamlNamespace);
            ApplyLocalProperties(xamlElement, hashtable, true);
            for (XmlNode node = htmlElement.FirstChild; node != null; node = node.NextSibling)
            {
                AddInline(xamlElement, node, hashtable2, stylesheet, sourceContext);
            }
            xamlParentElement.AppendChild(xamlElement);
        }

        private static void AddSection(XmlElement xamlParentElement, XmlElement htmlElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            XmlNode node;
            bool flag = false;
            for (node = htmlElement.FirstChild; node != null; node = node.NextSibling)
            {
                if ((node is XmlElement) && HtmlSchema.IsBlockElement(((XmlElement) node).LocalName.ToLower()))
                {
                    flag = true;
                    break;
                }
            }
            if (!flag)
            {
                AddParagraph(xamlParentElement, htmlElement, inheritedProperties, stylesheet, sourceContext);
            }
            else
            {
                Hashtable hashtable;
                Hashtable hashtable2 = GetElementProperties(htmlElement, inheritedProperties, out hashtable, stylesheet, sourceContext);
                XmlElement xamlElement = xamlParentElement.OwnerDocument.CreateElement(null, "Section", _xamlNamespace);
                ApplyLocalProperties(xamlElement, hashtable, true);
                if (!xamlElement.HasAttributes)
                {
                    xamlElement = xamlParentElement;
                }
                for (node = htmlElement.FirstChild; node != null; node = (node != null) ? node.NextSibling : null)
                {
                    node = AddBlock(xamlElement, node, hashtable2, stylesheet, sourceContext);
                }
                if (xamlElement != xamlParentElement)
                {
                    xamlParentElement.AppendChild(xamlElement);
                }
            }
        }

        private static void AddSpanOrRun(XmlElement xamlParentElement, XmlElement htmlElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Hashtable hashtable;
            bool flag = false;
            for (XmlNode node = htmlElement.FirstChild; node != null; node = node.NextSibling)
            {
                if (node is XmlElement)
                {
                    string xmlElementName = ((XmlElement) node).LocalName.ToLower();
                    if (((HtmlSchema.IsInlineElement(xmlElementName) || HtmlSchema.IsBlockElement(xmlElementName)) || ((xmlElementName == "img") || (xmlElementName == "br"))) || (xmlElementName == "hr"))
                    {
                        flag = true;
                        break;
                    }
                }
            }
            string localName = flag ? "Span" : "Run";
            Hashtable hashtable2 = GetElementProperties(htmlElement, inheritedProperties, out hashtable, stylesheet, sourceContext);
            XmlElement xamlElement = xamlParentElement.OwnerDocument.CreateElement(null, localName, _xamlNamespace);
            ApplyLocalProperties(xamlElement, hashtable, false);
            for (XmlNode node2 = htmlElement.FirstChild; node2 != null; node2 = node2.NextSibling)
            {
                AddInline(xamlElement, node2, hashtable2, stylesheet, sourceContext);
            }
            xamlParentElement.AppendChild(xamlElement);
        }

        private static void AddTable(XmlElement xamlParentElement, XmlElement htmlTableElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Hashtable hashtable;
            XmlNode firstChild;
            Debug.Assert(htmlTableElement.LocalName.ToLower() == "table");
            Debug.Assert(xamlParentElement != null);
            Debug.Assert(inheritedProperties != null);
            Hashtable hashtable2 = GetElementProperties(htmlTableElement, inheritedProperties, out hashtable, stylesheet, sourceContext);
            XmlElement cellFromSingleCellTable = GetCellFromSingleCellTable(htmlTableElement);
            if (cellFromSingleCellTable != null)
            {
                sourceContext.Add(cellFromSingleCellTable);
                for (firstChild = cellFromSingleCellTable.FirstChild; firstChild != null; firstChild = (firstChild != null) ? firstChild.NextSibling : null)
                {
                    firstChild = AddBlock(xamlParentElement, firstChild, hashtable2, stylesheet, sourceContext);
                }
                Debug.Assert((sourceContext.Count > 0) && (sourceContext[sourceContext.Count - 1] == cellFromSingleCellTable));
                sourceContext.RemoveAt(sourceContext.Count - 1);
            }
            else
            {
                XmlElement xamlTableElement = xamlParentElement.OwnerDocument.CreateElement(null, "Table", _xamlNamespace);
                ArrayList columnStartsAllRows = AnalyzeTableStructure(htmlTableElement, stylesheet);
                AddColumnInformation(htmlTableElement, xamlTableElement, columnStartsAllRows, hashtable2, stylesheet, sourceContext);
                firstChild = htmlTableElement.FirstChild;
                while (firstChild != null)
                {
                    XmlElement element3;
                    switch (firstChild.LocalName.ToLower())
                    {
                        case "tbody":
                        case "thead":
                        case "tfoot":
                        {
                            Hashtable hashtable3;
                            element3 = xamlTableElement.OwnerDocument.CreateElement(null, "TableRowGroup", _xamlNamespace);
                            xamlTableElement.AppendChild(element3);
                            sourceContext.Add((XmlElement) firstChild);
                            Hashtable currentProperties = GetElementProperties((XmlElement) firstChild, hashtable2, out hashtable3, stylesheet, sourceContext);
                            AddTableRowsToTableBody(element3, firstChild.FirstChild, currentProperties, columnStartsAllRows, stylesheet, sourceContext);
                            if (element3.HasChildNodes)
                            {
                                xamlTableElement.AppendChild(element3);
                            }
                            Debug.Assert((sourceContext.Count > 0) && (sourceContext[sourceContext.Count - 1] == firstChild));
                            sourceContext.RemoveAt(sourceContext.Count - 1);
                            firstChild = firstChild.NextSibling;
                            break;
                        }
                        case "tr":
                            element3 = xamlTableElement.OwnerDocument.CreateElement(null, "TableRowGroup", _xamlNamespace);
                            firstChild = AddTableRowsToTableBody(element3, firstChild, hashtable2, columnStartsAllRows, stylesheet, sourceContext);
                            if (element3.HasChildNodes)
                            {
                                xamlTableElement.AppendChild(element3);
                            }
                            break;

                        default:
                            firstChild = firstChild.NextSibling;
                            break;
                    }
                }
                if (xamlTableElement.HasChildNodes)
                {
                    xamlParentElement.AppendChild(xamlTableElement);
                }
            }
        }

        private static XmlNode AddTableCellsToTableRow(XmlElement xamlTableRowElement, XmlNode htmlTDStartNode, Hashtable currentProperties, ArrayList columnStarts, ArrayList activeRowSpans, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Debug.Assert(xamlTableRowElement.LocalName == "TableRow");
            Debug.Assert(currentProperties != null);
            if (columnStarts != null)
            {
                Debug.Assert(activeRowSpans.Count == columnStarts.Count);
            }
            XmlNode nextSibling = htmlTDStartNode;
            double num = 0.0;
            double columnWidth = 0.0;
            int columnIndex = 0;
            int num4 = 0;
            while ((((nextSibling != null) && (nextSibling.LocalName.ToLower() != "tr")) && ((nextSibling.LocalName.ToLower() != "tbody") && (nextSibling.LocalName.ToLower() != "thead"))) && (nextSibling.LocalName.ToLower() != "tfoot"))
            {
                if ((nextSibling.LocalName.ToLower() == "td") || (nextSibling.LocalName.ToLower() == "th"))
                {
                    Hashtable hashtable;
                    XmlElement xamlTableCellElement = xamlTableRowElement.OwnerDocument.CreateElement(null, "TableCell", _xamlNamespace);
                    sourceContext.Add((XmlElement) nextSibling);
                    Hashtable hashtable2 = GetElementProperties((XmlElement) nextSibling, currentProperties, out hashtable, stylesheet, sourceContext);
                    ApplyPropertiesToTableCellElement((XmlElement) nextSibling, xamlTableCellElement);
                    if (columnStarts != null)
                    {
                        Debug.Assert(columnIndex < (columnStarts.Count - 1));
                        while ((columnIndex < activeRowSpans.Count) && (((int) activeRowSpans[columnIndex]) > 0))
                        {
                            activeRowSpans[columnIndex] = ((int) activeRowSpans[columnIndex]) - 1;
                            Debug.Assert(((int) activeRowSpans[columnIndex]) >= 0);
                            columnIndex++;
                        }
                        Debug.Assert(columnIndex < (columnStarts.Count - 1));
                        num = (double) columnStarts[columnIndex];
                        columnWidth = GetColumnWidth((XmlElement) nextSibling);
                        num4 = CalculateColumnSpan(columnIndex, columnWidth, columnStarts);
                        int rowSpan = GetRowSpan((XmlElement) nextSibling);
                        Debug.Assert(num4 > 0);
                        Debug.Assert((columnIndex + num4) < columnStarts.Count);
                        xamlTableCellElement.SetAttribute("ColumnSpan", num4.ToString());
                        for (int i = columnIndex; i < (columnIndex + num4); i++)
                        {
                            Debug.Assert(i < activeRowSpans.Count);
                            activeRowSpans[i] = rowSpan - 1;
                            Debug.Assert(((int) activeRowSpans[i]) >= 0);
                        }
                        columnIndex += num4;
                    }
                    AddDataToTableCell(xamlTableCellElement, nextSibling.FirstChild, hashtable2, stylesheet, sourceContext);
                    if (xamlTableCellElement.HasChildNodes)
                    {
                        xamlTableRowElement.AppendChild(xamlTableCellElement);
                    }
                    Debug.Assert((sourceContext.Count > 0) && (sourceContext[sourceContext.Count - 1] == nextSibling));
                    sourceContext.RemoveAt(sourceContext.Count - 1);
                    nextSibling = nextSibling.NextSibling;
                }
                else
                {
                    nextSibling = nextSibling.NextSibling;
                }
            }
            return nextSibling;
        }

        private static void AddTableColumn(XmlElement xamlTableElement, XmlElement htmlColElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Hashtable hashtable;
            Hashtable hashtable2 = GetElementProperties(htmlColElement, inheritedProperties, out hashtable, stylesheet, sourceContext);
            XmlElement newChild = xamlTableElement.OwnerDocument.CreateElement(null, "TableColumn", _xamlNamespace);
            xamlTableElement.AppendChild(newChild);
        }

        private static void AddTableColumnGroup(XmlElement xamlTableElement, XmlElement htmlColgroupElement, Hashtable inheritedProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Hashtable hashtable;
            Hashtable hashtable2 = GetElementProperties(htmlColgroupElement, inheritedProperties, out hashtable, stylesheet, sourceContext);
            for (XmlNode node = htmlColgroupElement.FirstChild; node != null; node = node.NextSibling)
            {
                if ((node is XmlElement) && (node.LocalName.ToLower() == "col"))
                {
                    AddTableColumn(xamlTableElement, (XmlElement) node, hashtable2, stylesheet, sourceContext);
                }
            }
        }

        private static XmlNode AddTableRowsToTableBody(XmlElement xamlTableBodyElement, XmlNode htmlTRStartNode, Hashtable currentProperties, ArrayList columnStarts, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Debug.Assert(xamlTableBodyElement.LocalName == "TableRowGroup");
            Debug.Assert(currentProperties != null);
            XmlNode htmlTDStartNode = htmlTRStartNode;
            ArrayList activeRowSpans = null;
            if (columnStarts != null)
            {
                activeRowSpans = new ArrayList();
                InitializeActiveRowSpans(activeRowSpans, columnStarts.Count);
            }
            while ((htmlTDStartNode != null) && (htmlTDStartNode.LocalName.ToLower() != "tbody"))
            {
                XmlElement element;
                if (htmlTDStartNode.LocalName.ToLower() == "tr")
                {
                    Hashtable hashtable;
                    element = xamlTableBodyElement.OwnerDocument.CreateElement(null, "TableRow", _xamlNamespace);
                    sourceContext.Add((XmlElement) htmlTDStartNode);
                    Hashtable hashtable2 = GetElementProperties((XmlElement) htmlTDStartNode, currentProperties, out hashtable, stylesheet, sourceContext);
                    AddTableCellsToTableRow(element, htmlTDStartNode.FirstChild, hashtable2, columnStarts, activeRowSpans, stylesheet, sourceContext);
                    if (element.HasChildNodes)
                    {
                        xamlTableBodyElement.AppendChild(element);
                    }
                    Debug.Assert((sourceContext.Count > 0) && (sourceContext[sourceContext.Count - 1] == htmlTDStartNode));
                    sourceContext.RemoveAt(sourceContext.Count - 1);
                    htmlTDStartNode = htmlTDStartNode.NextSibling;
                }
                else if (htmlTDStartNode.LocalName.ToLower() == "td")
                {
                    element = xamlTableBodyElement.OwnerDocument.CreateElement(null, "TableRow", _xamlNamespace);
                    Debug.Assert(columnStarts == null);
                    htmlTDStartNode = AddTableCellsToTableRow(element, htmlTDStartNode, currentProperties, columnStarts, activeRowSpans, stylesheet, sourceContext);
                    if (element.HasChildNodes)
                    {
                        xamlTableBodyElement.AppendChild(element);
                    }
                }
                else
                {
                    htmlTDStartNode = htmlTDStartNode.NextSibling;
                }
            }
            return htmlTDStartNode;
        }

        private static void AddTextRun(XmlElement xamlElement, string textData)
        {
            for (int i = 0; i < textData.Length; i++)
            {
                if (char.IsControl(textData[i]))
                {
                    textData = textData.Remove(i--, 1);
                }
            }
            textData = textData.Replace('\x00a0', ' ');
            if (textData.Length > 0)
            {
                xamlElement.AppendChild(xamlElement.OwnerDocument.CreateTextNode(textData));
            }
        }

        private static ArrayList AnalyzeTableStructure(XmlElement htmlTableElement, CssStylesheet stylesheet)
        {
            Debug.Assert(htmlTableElement.LocalName.ToLower() == "table");
            if (!htmlTableElement.HasChildNodes)
            {
                return null;
            }
            bool flag = true;
            ArrayList columnStarts = new ArrayList();
            ArrayList activeRowSpans = new ArrayList();
            Debug.Assert(columnStarts.Count == activeRowSpans.Count);
            XmlNode firstChild = htmlTableElement.FirstChild;
            double tableWidth = 0.0;
            while ((firstChild != null) && flag)
            {
                double num3;
                Debug.Assert(columnStarts.Count == activeRowSpans.Count);
                string str = firstChild.LocalName.ToLower();
                if (str != null)
                {
                    if (!(str == "tbody"))
                    {
                        if (str == "tr")
                        {
                            goto Label_010A;
                        }
                        if (str == "td")
                        {
                            goto Label_014F;
                        }
                    }
                    else
                    {
                        double num2 = AnalyzeTbodyStructure((XmlElement) firstChild, columnStarts, activeRowSpans, tableWidth, stylesheet);
                        if (num2 > tableWidth)
                        {
                            tableWidth = num2;
                        }
                        else if (num2 == 0.0)
                        {
                            flag = false;
                        }
                    }
                }
                goto Label_0155;
            Label_010A:
                num3 = AnalyzeTRStructure((XmlElement) firstChild, columnStarts, activeRowSpans, tableWidth, stylesheet);
                if (num3 > tableWidth)
                {
                    tableWidth = num3;
                }
                else if (num3 == 0.0)
                {
                    flag = false;
                }
                goto Label_0155;
            Label_014F:
                flag = false;
            Label_0155:
                firstChild = firstChild.NextSibling;
            }
            if (flag)
            {
                columnStarts.Add(tableWidth);
                VerifyColumnStartsAscendingOrder(columnStarts);
            }
            else
            {
                columnStarts = null;
            }
            return columnStarts;
        }

        private static double AnalyzeTbodyStructure(XmlElement htmlTbodyElement, ArrayList columnStarts, ArrayList activeRowSpans, double tableWidth, CssStylesheet stylesheet)
        {
            Debug.Assert(htmlTbodyElement.LocalName.ToLower() == "tbody");
            Debug.Assert(columnStarts != null);
            double num = 0.0;
            bool flag = true;
            if (!htmlTbodyElement.HasChildNodes)
            {
                return num;
            }
            ClearActiveRowSpans(activeRowSpans);
            for (XmlNode node = htmlTbodyElement.FirstChild; (node != null) && flag; node = node.NextSibling)
            {
                string str = node.LocalName.ToLower();
                if (str != null)
                {
                    if (!(str == "tr"))
                    {
                        if (str == "td")
                        {
                            goto Label_00AE;
                        }
                    }
                    else
                    {
                        double num2 = AnalyzeTRStructure((XmlElement) node, columnStarts, activeRowSpans, num, stylesheet);
                        if (num2 > num)
                        {
                            num = num2;
                        }
                    }
                }
                continue;
            Label_00AE:
                flag = false;
            }
            ClearActiveRowSpans(activeRowSpans);
            return (flag ? num : 0.0);
        }

        private static double AnalyzeTRStructure(XmlElement htmlTRElement, ArrayList columnStarts, ArrayList activeRowSpans, double tableWidth, CssStylesheet stylesheet)
        {
            Debug.Assert(htmlTRElement.LocalName.ToLower() == "tr");
            Debug.Assert(columnStarts != null);
            Debug.Assert(activeRowSpans != null);
            Debug.Assert(columnStarts.Count == activeRowSpans.Count);
            if (!htmlTRElement.HasChildNodes)
            {
                return 0.0;
            }
            bool flag = true;
            double num2 = 0.0;
            XmlNode firstChild = htmlTRElement.FirstChild;
            int index = 0;
            double num4 = 0.0;
            if (index < activeRowSpans.Count)
            {
                Debug.Assert(((double) columnStarts[index]) >= num2);
                if (((double) columnStarts[index]) == num2)
                {
                    while ((index < activeRowSpans.Count) && (((int) activeRowSpans[index]) > 0))
                    {
                        activeRowSpans[index] = ((int) activeRowSpans[index]) - 1;
                        Debug.Assert(((int) activeRowSpans[index]) >= 0);
                        index++;
                        num2 = (double) columnStarts[index];
                    }
                }
            }
            while ((firstChild != null) && flag)
            {
                Debug.Assert(columnStarts.Count == activeRowSpans.Count);
                VerifyColumnStartsAscendingOrder(columnStarts);
                string str = firstChild.LocalName.ToLower();
                if ((str != null) && (str == "td"))
                {
                    Debug.Assert(index <= columnStarts.Count);
                    if (index < columnStarts.Count)
                    {
                        Debug.Assert(num2 <= ((double) columnStarts[index]));
                        if (num2 < ((double) columnStarts[index]))
                        {
                            columnStarts.Insert(index, num2);
                            activeRowSpans.Insert(index, 0);
                        }
                    }
                    else
                    {
                        columnStarts.Add(num2);
                        activeRowSpans.Add(0);
                    }
                    double columnWidth = GetColumnWidth((XmlElement) firstChild);
                    if (columnWidth != -1.0)
                    {
                        int rowSpan = GetRowSpan((XmlElement) firstChild);
                        int num5 = GetNextColumnIndex(index, columnWidth, columnStarts, activeRowSpans);
                        if (num5 != -1)
                        {
                            Debug.Assert(num5 <= columnStarts.Count);
                            for (int i = index; i < num5; i++)
                            {
                                activeRowSpans[i] = rowSpan - 1;
                                Debug.Assert(((int) activeRowSpans[i]) >= 0);
                            }
                            index = num5;
                            num2 += columnWidth;
                            if (index < activeRowSpans.Count)
                            {
                                Debug.Assert(((double) columnStarts[index]) >= num2);
                                if (((double) columnStarts[index]) == num2)
                                {
                                    while ((index < activeRowSpans.Count) && (((int) activeRowSpans[index]) > 0))
                                    {
                                        activeRowSpans[index] = ((int) activeRowSpans[index]) - 1;
                                        Debug.Assert(((int) activeRowSpans[index]) >= 0);
                                        index++;
                                        num2 = (double) columnStarts[index];
                                    }
                                }
                            }
                        }
                        else
                        {
                            flag = false;
                        }
                    }
                    else
                    {
                        flag = false;
                    }
                }
                firstChild = firstChild.NextSibling;
            }
            if (flag)
            {
                num4 = num2;
            }
            else
            {
                num4 = 0.0;
            }
            return num4;
        }

        private static void ApplyLocalProperties(XmlElement xamlElement, Hashtable localProperties, bool isBlock)
        {
            bool flag = false;
            string top = "0";
            string bottom = "0";
            string left = "0";
            string right = "0";
            bool flag2 = false;
            string str5 = "0";
            string str6 = "0";
            string str7 = "0";
            string str8 = "0";
            string str9 = null;
            bool flag3 = false;
            string str10 = "0";
            string str11 = "0";
            string str12 = "0";
            string str13 = "0";
            IDictionaryEnumerator enumerator = localProperties.GetEnumerator();
            while (enumerator.MoveNext())
            {
                switch (((string) enumerator.Key))
                {
                    case "font-family":
                    {
                        xamlElement.SetAttribute("FontFamily", (string) enumerator.Value);
                        continue;
                    }
                    case "font-style":
                    {
                        xamlElement.SetAttribute("FontStyle", (string) enumerator.Value);
                        continue;
                    }
                    case "font-variant":
                    case "text-transform":
                    case "width":
                    case "height":
                    case "border-style-top":
                    case "border-style-right":
                    case "border-style-bottom":
                    case "border-style-left":
                    case "display":
                    {
                        continue;
                    }
                    case "font-weight":
                    {
                        xamlElement.SetAttribute("FontWeight", (string) enumerator.Value);
                        continue;
                    }
                    case "font-size":
                    {
                        xamlElement.SetAttribute("FontSize", (string) enumerator.Value);
                        continue;
                    }
                    case "color":
                    {
                        SetPropertyValue(xamlElement, TextElement.ForegroundProperty, (string) enumerator.Value);
                        continue;
                    }
                    case "background-color":
                    {
                        SetPropertyValue(xamlElement, TextElement.BackgroundProperty, (string) enumerator.Value);
                        continue;
                    }
                    case "text-decoration-underline":
                    {
                        if (!isBlock && (((string) enumerator.Value) == "true"))
                        {
                            xamlElement.SetAttribute("TextDecorations", "Underline");
                        }
                        continue;
                    }
                    case "text-decoration-none":
                    case "text-decoration-overline":
                    case "text-decoration-line-through":
                    case "text-decoration-blink":
                    {
                        if (isBlock)
                        {
                        }
                        continue;
                    }
                    case "text-indent":
                    {
                        if (isBlock)
                        {
                            xamlElement.SetAttribute("TextIndent", (string) enumerator.Value);
                        }
                        continue;
                    }
                    case "text-align":
                    {
                        if (isBlock)
                        {
                            xamlElement.SetAttribute("TextAlignment", (string) enumerator.Value);
                        }
                        continue;
                    }
                    case "margin-top":
                    {
                        flag = true;
                        top = (string) enumerator.Value;
                        continue;
                    }
                    case "margin-right":
                    {
                        flag = true;
                        right = (string) enumerator.Value;
                        continue;
                    }
                    case "margin-bottom":
                    {
                        flag = true;
                        bottom = (string) enumerator.Value;
                        continue;
                    }
                    case "margin-left":
                    {
                        flag = true;
                        left = (string) enumerator.Value;
                        continue;
                    }
                    case "padding-top":
                    {
                        flag2 = true;
                        str5 = (string) enumerator.Value;
                        continue;
                    }
                    case "padding-right":
                    {
                        flag2 = true;
                        str8 = (string) enumerator.Value;
                        continue;
                    }
                    case "padding-bottom":
                    {
                        flag2 = true;
                        str6 = (string) enumerator.Value;
                        continue;
                    }
                    case "padding-left":
                    {
                        flag2 = true;
                        str7 = (string) enumerator.Value;
                        continue;
                    }
                    case "border-color-top":
                    {
                        str9 = (string) enumerator.Value;
                        continue;
                    }
                    case "border-color-right":
                    {
                        str9 = (string) enumerator.Value;
                        continue;
                    }
                    case "border-color-bottom":
                    {
                        str9 = (string) enumerator.Value;
                        continue;
                    }
                    case "border-color-left":
                    {
                        str9 = (string) enumerator.Value;
                        continue;
                    }
                    case "border-width-top":
                    {
                        flag3 = true;
                        str10 = (string) enumerator.Value;
                        continue;
                    }
                    case "border-width-right":
                    {
                        flag3 = true;
                        str13 = (string) enumerator.Value;
                        continue;
                    }
                    case "border-width-bottom":
                    {
                        flag3 = true;
                        str11 = (string) enumerator.Value;
                        continue;
                    }
                    case "border-width-left":
                    {
                        flag3 = true;
                        str12 = (string) enumerator.Value;
                        continue;
                    }
                    case "list-style-type":
                        break;

                    case "float":
                    case "clear":
                    {
                        if (!isBlock)
                        {
                        }
                        continue;
                    }
                    default:
                    {
                        continue;
                    }
                }
                if (xamlElement.LocalName == "List")
                {
                    string str14;
                    switch (((string) enumerator.Value).ToLower())
                    {
                        case "disc":
                            str14 = "Disc";
                            break;

                        case "circle":
                            str14 = "Circle";
                            break;

                        case "none":
                            str14 = "None";
                            break;

                        case "square":
                            str14 = "Square";
                            break;

                        case "box":
                            str14 = "Box";
                            break;

                        case "lower-latin":
                            str14 = "LowerLatin";
                            break;

                        case "upper-latin":
                            str14 = "UpperLatin";
                            break;

                        case "lower-roman":
                            str14 = "LowerRoman";
                            break;

                        case "upper-roman":
                            str14 = "UpperRoman";
                            break;

                        case "decimal":
                            str14 = "Decimal";
                            break;

                        default:
                            str14 = "Disc";
                            break;
                    }
                    xamlElement.SetAttribute("MarkerStyle", str14);
                }
            }
            if (isBlock)
            {
                if (flag)
                {
                    ComposeThicknessProperty(xamlElement, "Margin", left, right, top, bottom);
                }
                if (flag2)
                {
                    ComposeThicknessProperty(xamlElement, "Padding", str7, str8, str5, str6);
                }
                if (str9 != null)
                {
                    xamlElement.SetAttribute("BorderBrush", str9);
                }
                if (flag3)
                {
                    ComposeThicknessProperty(xamlElement, "BorderThickness", str12, str13, str10, str11);
                }
            }
        }

        private static void ApplyPropertiesToTableCellElement(XmlElement htmlChildNode, XmlElement xamlTableCellElement)
        {
            Debug.Assert((htmlChildNode.LocalName.ToLower() == "td") || (htmlChildNode.LocalName.ToLower() == "th"));
            Debug.Assert(xamlTableCellElement.LocalName == "TableCell");
            xamlTableCellElement.SetAttribute("BorderThickness", "1,1,1,1");
            xamlTableCellElement.SetAttribute("BorderBrush", "Black");
            string attribute = GetAttribute(htmlChildNode, "rowspan");
            if (attribute != null)
            {
                xamlTableCellElement.SetAttribute("RowSpan", attribute);
            }
        }

        private static int CalculateColumnSpan(int columnIndex, double columnWidth, ArrayList columnStarts)
        {
            Debug.Assert(columnStarts != null);
            Debug.Assert(columnIndex < (columnStarts.Count - 1));
            Debug.Assert(((double) columnStarts[columnIndex]) >= 0.0);
            Debug.Assert(columnWidth > 0.0);
            int num2 = columnIndex;
            double num = 0.0;
            int num3 = 0;
            double num4 = 0.0;
            while ((num < columnWidth) && (num2 < (columnStarts.Count - 1)))
            {
                num4 = ((double) columnStarts[num2 + 1]) - ((double) columnStarts[num2]);
                Debug.Assert(num4 > 0.0);
                num += num4;
                num2++;
            }
            num3 = num2 - columnIndex;
            Debug.Assert(num3 > 0);
            return num3;
        }

        private static void ClearActiveRowSpans(ArrayList activeRowSpans)
        {
            for (int i = 0; i < activeRowSpans.Count; i++)
            {
                activeRowSpans[i] = 0;
            }
        }

        private static void ComposeThicknessProperty(XmlElement xamlElement, string propertyName, string left, string right, string top, string bottom)
        {
            string str;
            if ((left[0] == '0') || (left[0] == '-'))
            {
                left = "0";
            }
            if ((right[0] == '0') || (right[0] == '-'))
            {
                right = "0";
            }
            if ((top[0] == '0') || (top[0] == '-'))
            {
                top = "0";
            }
            if ((bottom[0] == '0') || (bottom[0] == '-'))
            {
                bottom = "0";
            }
            if ((left == right) && (top == bottom))
            {
                if (left == top)
                {
                    str = left;
                }
                else
                {
                    str = left + "," + top;
                }
            }
            else
            {
                str = left + "," + top + "," + right + "," + bottom;
            }
            xamlElement.SetAttribute(propertyName, str);
        }

        public static string ConvertHtmlToXaml(string htmlString, bool asFlowDocument)
        {
            XmlElement htmlElement = HtmlParser.ParseHtml(htmlString);
            string localName = asFlowDocument ? "FlowDocument" : "Section";
            XmlElement xamlParentElement = new XmlDocument().CreateElement(null, localName, _xamlNamespace);
            CssStylesheet stylesheet = new CssStylesheet(htmlElement);
            List<XmlElement> sourceContext = new List<XmlElement>(10);
            InlineFragmentParentElement = null;
            AddBlock(xamlParentElement, htmlElement, new Hashtable(), stylesheet, sourceContext);
            if (!asFlowDocument)
            {
                xamlParentElement = ExtractInlineFragment(xamlParentElement);
            }
            xamlParentElement.SetAttribute("xml:space", "preserve");
            return xamlParentElement.OuterXml;
        }

        private static void DefineInlineFragmentParent(XmlComment htmlComment, XmlElement xamlParentElement)
        {
            if (htmlComment.Value == "StartFragment")
            {
                InlineFragmentParentElement = xamlParentElement;
            }
            else if ((htmlComment.Value == "EndFragment") && ((InlineFragmentParentElement == null) && (xamlParentElement != null)))
            {
                InlineFragmentParentElement = xamlParentElement;
            }
        }

        private static XmlElement ExtractInlineFragment(XmlElement xamlFlowDocumentElement)
        {
            if (InlineFragmentParentElement != null)
            {
                if (InlineFragmentParentElement.LocalName == "Span")
                {
                    xamlFlowDocumentElement = InlineFragmentParentElement;
                    return xamlFlowDocumentElement;
                }
                xamlFlowDocumentElement = xamlFlowDocumentElement.OwnerDocument.CreateElement(null, "Span", _xamlNamespace);
                while (InlineFragmentParentElement.FirstChild != null)
                {
                    XmlNode firstChild = InlineFragmentParentElement.FirstChild;
                    InlineFragmentParentElement.RemoveChild(firstChild);
                    xamlFlowDocumentElement.AppendChild(firstChild);
                }
            }
            return xamlFlowDocumentElement;
        }

        public static string GetAttribute(XmlElement element, string attributeName)
        {
            attributeName = attributeName.ToLower();
            for (int i = 0; i < element.Attributes.Count; i++)
            {
                if (element.Attributes[i].Name.ToLower() == attributeName)
                {
                    return element.Attributes[i].Value;
                }
            }
            return null;
        }

        private static XmlElement GetCellFromSingleCellTable(XmlElement htmlTableElement)
        {
            XmlElement element = null;
            for (XmlNode node = htmlTableElement.FirstChild; node != null; node = node.NextSibling)
            {
                XmlNode firstChild;
                switch (node.LocalName.ToLower())
                {
                    case "tbody":
                    case "thead":
                    case "tfoot":
                        if (element != null)
                        {
                            return null;
                        }
                        for (XmlNode node2 = node.FirstChild; node2 != null; node2 = node2.NextSibling)
                        {
                            if (node2.LocalName.ToLower() == "tr")
                            {
                                if (element != null)
                                {
                                    return null;
                                }
                                firstChild = node2.FirstChild;
                                while (firstChild != null)
                                {
                                    switch (firstChild.LocalName.ToLower())
                                    {
                                        case "td":
                                        case "th":
                                            if (element != null)
                                            {
                                                return null;
                                            }
                                            element = (XmlElement) firstChild;
                                            break;
                                    }
                                    firstChild = firstChild.NextSibling;
                                }
                            }
                        }
                        break;

                    default:
                        if (node.LocalName.ToLower() == "tr")
                        {
                            if (element != null)
                            {
                                return null;
                            }
                            for (firstChild = node.FirstChild; firstChild != null; firstChild = firstChild.NextSibling)
                            {
                                switch (firstChild.LocalName.ToLower())
                                {
                                    case "td":
                                    case "th":
                                        if (element != null)
                                        {
                                            return null;
                                        }
                                        element = (XmlElement) firstChild;
                                        break;
                                }
                            }
                        }
                        break;
                }
            }
            return element;
        }

        private static string GetColorValue(string colorValue)
        {
            return colorValue;
        }

        private static double GetColumnWidth(XmlElement htmlTDElement)
        {
            string lengthAsString = null;
            double length = -1.0;
            lengthAsString = GetAttribute(htmlTDElement, "width");
            if (lengthAsString == null)
            {
                lengthAsString = GetCssAttribute(GetAttribute(htmlTDElement, "style"), "width");
            }
            if (!(TryGetLengthValue(lengthAsString, out length) && (length != 0.0)))
            {
                length = -1.0;
            }
            return length;
        }

        private static string GetCssAttribute(string cssStyle, string attributeName)
        {
            if (cssStyle != null)
            {
                attributeName = attributeName.ToLower();
                string[] strArray = cssStyle.Split(new char[] { ';' });
                for (int i = 0; i < strArray.Length; i++)
                {
                    string[] strArray2 = strArray[i].Split(new char[] { ':' });
                    if ((strArray2.Length == 2) && (strArray2[0].Trim().ToLower() == attributeName))
                    {
                        return strArray2[1].Trim();
                    }
                }
            }
            return null;
        }

        private static Hashtable GetElementProperties(XmlElement htmlElement, Hashtable inheritedProperties, out Hashtable localProperties, CssStylesheet stylesheet, List<XmlElement> sourceContext)
        {
            Hashtable hashtable = new Hashtable();
            IDictionaryEnumerator enumerator = inheritedProperties.GetEnumerator();
            while (enumerator.MoveNext())
            {
                hashtable[enumerator.Key] = enumerator.Value;
            }
            string elementName = htmlElement.LocalName.ToLower();
            string namespaceURI = htmlElement.NamespaceURI;
            localProperties = new Hashtable();
            switch (elementName)
            {
                case "i":
                case "italic":
                case "em":
                    localProperties["font-style"] = "italic";
                    break;

                case "b":
                case "bold":
                case "strong":
                case "dfn":
                    localProperties["font-weight"] = "bold";
                    break;

                case "u":
                case "underline":
                    localProperties["text-decoration-underline"] = "true";
                    break;

                case "font":
                {
                    string attribute = GetAttribute(htmlElement, "face");
                    if (attribute != null)
                    {
                        localProperties["font-family"] = attribute;
                    }
                    attribute = GetAttribute(htmlElement, "size");
                    if (attribute != null)
                    {
                        double num = double.Parse(attribute) * 4.0;
                        if (num < 1.0)
                        {
                            num = 1.0;
                        }
                        else if (num > 1000.0)
                        {
                            num = 1000.0;
                        }
                        localProperties["font-size"] = num.ToString();
                    }
                    attribute = GetAttribute(htmlElement, "color");
                    if (attribute != null)
                    {
                        localProperties["color"] = attribute;
                    }
                    break;
                }
                case "samp":
                    localProperties["font-family"] = "Courier New";
                    localProperties["font-size"] = "8pt";
                    localProperties["text-align"] = "Left";
                    break;

                case "pre":
                    localProperties["font-family"] = "Courier New";
                    localProperties["font-size"] = "8pt";
                    localProperties["text-align"] = "Left";
                    break;

                case "blockquote":
                    localProperties["margin-left"] = "16";
                    break;

                case "h1":
                    localProperties["font-size"] = "22pt";
                    break;

                case "h2":
                    localProperties["font-size"] = "20pt";
                    break;

                case "h3":
                    localProperties["font-size"] = "18pt";
                    break;

                case "h4":
                    localProperties["font-size"] = "16pt";
                    break;

                case "h5":
                    localProperties["font-size"] = "12pt";
                    break;

                case "h6":
                    localProperties["font-size"] = "10pt";
                    break;

                case "ul":
                    localProperties["list-style-type"] = "disc";
                    break;

                case "ol":
                    localProperties["list-style-type"] = "decimal";
                    break;
            }
            HtmlCssParser.GetElementPropertiesFromCssAttributes(htmlElement, elementName, stylesheet, localProperties, sourceContext);
            enumerator = localProperties.GetEnumerator();
            while (enumerator.MoveNext())
            {
                hashtable[enumerator.Key] = enumerator.Value;
            }
            return hashtable;
        }

        private static int GetNextColumnIndex(int columnIndex, double columnWidth, ArrayList columnStarts, ArrayList activeRowSpans)
        {
            Debug.Assert(columnStarts != null);
            Debug.Assert((0 <= columnIndex) && (columnIndex <= columnStarts.Count));
            Debug.Assert(columnWidth > 0.0);
            double num = (double) columnStarts[columnIndex];
            int num2 = columnIndex + 1;
            while (((num2 < columnStarts.Count) && (((double) columnStarts[num2]) < (num + columnWidth))) && (num2 != -1))
            {
                if (((int) activeRowSpans[num2]) > 0)
                {
                    num2 = -1;
                }
                else
                {
                    num2++;
                }
            }
            return num2;
        }

        private static double GetNextColumnStart(XmlElement htmlTDElement, double columnStart)
        {
            Debug.Assert((htmlTDElement.LocalName.ToLower() == "td") || (htmlTDElement.LocalName.ToLower() == "th"));
            Debug.Assert(columnStart >= 0.0);
            double columnWidth = GetColumnWidth(htmlTDElement);
            if (columnWidth == -1.0)
            {
                return -1.0;
            }
            return (columnStart + columnWidth);
        }

        private static int GetRowSpan(XmlElement htmlTDElement)
        {
            int num;
            string attribute = GetAttribute(htmlTDElement, "rowspan");
            if (attribute == null)
            {
                return 1;
            }
            if (!int.TryParse(attribute, out num))
            {
                num = 1;
            }
            return num;
        }

        private static void InitializeActiveRowSpans(ArrayList activeRowSpans, int count)
        {
            for (int i = 0; i < count; i++)
            {
                activeRowSpans.Add(0);
            }
        }

        private static void SetPropertyValue(XmlElement xamlElement, DependencyProperty property, string stringValue)
        {
            TypeConverter converter = TypeDescriptor.GetConverter(property.PropertyType);
            try
            {
                if (converter.ConvertFromInvariantString(stringValue) != null)
                {
                    xamlElement.SetAttribute(property.Name, stringValue);
                }
            }
            catch (Exception)
            {
            }
        }

        private static bool TryGetLengthValue(string lengthAsString, out double length)
        {
            length = double.NaN;
            if (lengthAsString != null)
            {
                lengthAsString = lengthAsString.Trim().ToLower();
                if (lengthAsString.EndsWith("pt"))
                {
                    lengthAsString = lengthAsString.Substring(0, lengthAsString.Length - 2);
                    if (double.TryParse(lengthAsString, out length))
                    {
                        length = (length * 96.0) / 72.0;
                    }
                    else
                    {
                        length = double.NaN;
                    }
                }
                else if (lengthAsString.EndsWith("px"))
                {
                    lengthAsString = lengthAsString.Substring(0, lengthAsString.Length - 2);
                    if (!double.TryParse(lengthAsString, out length))
                    {
                        length = double.NaN;
                    }
                }
                else if (!double.TryParse(lengthAsString, out length))
                {
                    length = double.NaN;
                }
            }
            return !double.IsNaN(length);
        }

        internal static string UnQuote(string value)
        {
            if ((value.StartsWith("\"") && value.EndsWith("\"")) || (value.StartsWith("'") && value.EndsWith("'")))
            {
                value = value.Substring(1, value.Length - 2).Trim();
            }
            return value;
        }

        private static void VerifyColumnStartsAscendingOrder(ArrayList columnStarts)
        {
            Debug.Assert(columnStarts != null);
            double num = -0.01;
            for (int i = 0; i < columnStarts.Count; i++)
            {
                Debug.Assert(num < ((double) columnStarts[i]));
                num = (double) columnStarts[i];
            }
        }
    }
}

