﻿namespace NAVAddIns
{
    using Microsoft.Dynamics.Framework.UI.Extensibility;
    using Microsoft.Dynamics.Framework.UI.Extensibility.WinForms;
    using NAVEnhancedNote;
    using RecordLink;
    using System;
    using System.ComponentModel;
    using System.Drawing;
    using System.IO;
    using System.Runtime.CompilerServices;
    using System.Threading;
    using System.Windows.Forms;
    using System.Windows.Forms.Integration;
    using System.Xml;

    [Description("Enhanced notifications for the NAV Role Tailored Client."), ControlAddInExport("NAV Enhanced Notes")]
    public class NAVEnhancedNotesAddin : WinFormsControlAddInBase, IValueControlAddInDefinition<string>, IEventControlAddInDefinition
    {
        private ElementHost _elementHost;
        private string _email;
        private TextBox _errorTextBox;
        private string _lastValue;
        private EnhancedNotesControl _NAVcontrol;
        private string _style;
        private string _userid;
        private NAVRecordLink selectedLink;

        public event ControlAddInEventHandler ControlAddIn;

        private event controlcall formControlPointer;

        private event functioncall formFunctionPointer;

        protected override Control CreateControl()
        {
            try
            {
                this._NAVcontrol = new EnhancedNotesControl();
                if (this._NAVcontrol != null)
                {
                    this._NAVcontrol.NoteSaved += new EnhancedNotesControl.SaveTheNote(this.NoteSaved);
                    this._NAVcontrol.NoteChecked += new EnhancedNotesControl.CheckForNotes(this.NoteChecked);
                    this.formControlPointer += new controlcall(this.RemoveNote);
                    this._NAVcontrol.userControlPointer = this.formControlPointer;
                    this.formFunctionPointer += new functioncall(this.Replicate);
                    this._NAVcontrol.userFunctionPointer = this.formFunctionPointer;
                    ElementHost host = new ElementHost {
                        Dock = DockStyle.Fill,
                        MinimumSize = new Size((int) this._NAVcontrol.Width, (int) this._NAVcontrol.Height)
                    };
                    this._elementHost = host;
                    this._elementHost.Child = this._NAVcontrol;
                }
                return this._elementHost;
            }
            catch (Exception exception)
            {
                this._errorTextBox = new TextBox();
                this._errorTextBox.Width = 300;
                this._errorTextBox.Height = 600;
                this._errorTextBox.Multiline = true;
                this._errorTextBox.Text = "Error creating control in .NET Code:" + Environment.NewLine + Environment.NewLine + exception.Message + Environment.NewLine + Environment.NewLine + exception.StackTrace;
                return this._errorTextBox;
            }
        }

        private bool isExpectedValue(string value)
        {
            return ((value != null) && value.StartsWith("<XML"));
        }

        private void NoteChecked(EnhancedNotesControl m, EventArgs e)
        {
            string data = "<XML><Note action=\"RequestNew\" to=\"" + this._userid + "\" notify=\"Yes\" /></XML>";
            this.RaiseControlAddInEvent(2, data);
        }

        private void NoteSaved(EnhancedNotesControl m, EventArgs e)
        {
            XmlCDataSection section = new XmlDocument().CreateCDataSection(m.Note);
            string str = HtmlRemoval.StripTagsRegexCompiled(m.Note);
            string data = "<XML><Note action=\"New\" to=\"" + m.To + "\" notify=\"" + m.Notify + "\" text=\"" + str + "\" rtfnote=\"" + section.OuterXml + "\" /></XML>";
            this.RaiseControlAddInEvent(1, data);
        }

        private void ParseXML(string xmlString)
        {
            XmlReader reader = XmlReader.Create(new StringReader(xmlString));
            bool flag = false;
            while (reader.Read())
            {
                if (reader.NodeType == XmlNodeType.Element)
                {
                    if (reader.Name.Equals("User"))
                    {
                        if (this._NAVcontrol != null)
                        {
                            this._NAVcontrol.AddUser(reader.GetAttribute("name"), reader.GetAttribute("sid"));
                        }
                    }
                    else if (reader.Name.Equals("Note"))
                    {
                        flag = true;
                        if (this._NAVcontrol != null)
                        {
                            string[] strArray = new string[] { reader.GetAttribute("from"), reader.GetAttribute("to"), reader.GetAttribute("date"), reader.GetAttribute("text"), reader.GetAttribute("page"), reader.GetAttribute("linkid"), reader.GetAttribute("url"), reader.GetAttribute("parentLinkID"), reader.Value };
                            reader.Read();
                            strArray[8] = reader.Value;
                            NAVRecordLink note = new NAVRecordLink(strArray[0], strArray[1], strArray[2], strArray[3], strArray[4], strArray[5], strArray[6], strArray[7], strArray[8]);
                            this._NAVcontrol.AddNote(note);
                        }
                    }
                    else if (reader.Name.Equals("Me"))
                    {
                        this._userid = reader.GetAttribute("id");
                        this._email = reader.GetAttribute("email");
                        this._NAVcontrol.Email = this._email;
                    }
                    else if (reader.Name.Equals("Style"))
                    {
                        if (this._NAVcontrol != null)
                        {
                            this._style = reader.GetAttribute("type");
                            this._NAVcontrol.DisplayStyle = this._style;
                        }
                    }
                    else if (reader.Name.Equals("Popup") && (this._NAVcontrol != null))
                    {
                        this._NAVcontrol.CreatePopup(reader.GetAttribute("note"), reader.GetAttribute("url"));
                    }
                }
            }
            if (!(flag || !xmlString.StartsWith("<XML type=\"Notes\">")))
            {
                this._NAVcontrol.Clear();
            }
        }

        private void RaiseControlAddInEvent(int eventID, string data)
        {
            if (this.ControlAddIn != null)
            {
                this.ControlAddIn(eventID, data);
            }
        }

        private void RemoveNote(object sender, EventArgs e)
        {
            string data = "<XML><Note action=\"Remove\" linkid=\"" + this.selectedLink.linkID + "\" /></XML>";
            this.RaiseControlAddInEvent(3, data);
        }

        private void Replicate(NAVRecordLink link)
        {
            this.selectedLink = link;
        }

        public override bool AllowCaptionControl
        {
            get
            {
                return false;
            }
        }

        public bool HasValueChanged
        {
            get
            {
                return false;
            }
        }

        public string Value
        {
            get
            {
                return base.Control.Text;
            }
            set
            {
                if ((base.Control != null) && this.isExpectedValue(value))
                {
                    try
                    {
                        if (((this._lastValue == null) || !this._lastValue.Equals(value)) || value.StartsWith("<XML type=\"Notes\">"))
                        {
                            this.ParseXML(value);
                            this._lastValue = value;
                        }
                    }
                    catch (Exception exception)
                    {
                        MessageBox.Show("Error retrieving value in .NET Code:" + Environment.NewLine + Environment.NewLine + exception.Message + Environment.NewLine + Environment.NewLine + exception.StackTrace);
                    }
                    finally
                    {
                        if (this._NAVcontrol != null)
                        {
                            this._NAVcontrol.FinalizeDataLoad();
                            this._NAVcontrol.Refresh();
                        }
                    }
                }
            }
        }

        public delegate void controlcall(object sender, EventArgs e);

        public delegate void functioncall(NAVRecordLink message);
    }
}

