﻿namespace NAVEnhancedNote
{
    using Controls;
    using NAVPopup;
    using NodeList;
    using RecordLink;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Runtime.CompilerServices;
    using System.Threading;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Forms;
    using System.Windows.Forms.Integration;
    using System.Windows.Input;
    using System.Windows.Markup;
    using System.Windows.Shapes;
    using System.Windows.Threading;

    public class EnhancedNotesControl : System.Windows.Controls.UserControl, IComponentConnector
    {
        private BackgroundWorker _bw;
        private bool _contentLoaded;
        private string _defaultText;
        private string _myEmail;
        private bool _noteClosed;
        private string _style;
        internal System.Windows.Controls.Button AdvancedButton;
        internal System.Windows.Controls.Button CancelButton;
        public EventArgs e1 = null;
        public EventArgs e2 = null;
        internal NotesListBox ExistingNotesList;
        private static bool isRunning;
        private DateTime lastRunTime;
        private NAVRecordLink linkFromChild;
        internal MyNotes MyNotesList;
        internal System.Windows.Controls.TextBox NewNoteTextBox;
        internal System.Windows.Controls.TextBox NotesTextbox;
        internal System.Windows.Controls.CheckBox NotifyCheckBox;
        internal System.Windows.Controls.Label NotifyLabel;
        private ArrayList openPopups;
        internal Rectangle rectangle1;
        internal System.Windows.Controls.Button SaveButton;
        internal ComboWithCheckBoxes SendToComboBox;
        internal System.Windows.Controls.Label SendToLabel;
        private System.Windows.Forms.Timer timer;
        private int uniqueID;
        public Delegate userControlPointer;
        public Delegate userFunctionPointer;
        private ObservableNodeList userList;
        private Dictionary<string, Node> userNodeList;

        public event controlcall formControlPointer;

        public event functioncall formFunctionPointer;

        public event CheckForNotes NoteChecked;

        public event SaveTheNote NoteSaved;

        public EnhancedNotesControl()
        {
            this.InitializeComponent();
            this._defaultText = "Enter a new note here...";
            this.userList = new ObservableNodeList();
            this.userNodeList = new Dictionary<string, Node>();
            this._style = "";
            this.formControlPointer += new controlcall(this.RemoveNote);
            this.MyNotesList.userControlPointer = this.formControlPointer;
            this.formFunctionPointer += new functioncall(this.Replicate);
            this.MyNotesList.userFunctionPointer = this.formFunctionPointer;
            this.CloseNote();
        }

        public void AddNote(NAVRecordLink note)
        {
            if (this._style.Equals("MyNotes"))
            {
                this.MyNotesList.Add(note);
            }
            else
            {
                this.ExistingNotesList.Add(note);
            }
        }

        public void AddNote(NAVRecordLink note, int index)
        {
            if (!this._style.Equals("MyNotes"))
            {
                this.ExistingNotesList.Add(note, index);
            }
        }

        public void AddUser(string name, string sid)
        {
            if (!this.userNodeList.ContainsKey(name))
            {
                Node item = new Node(name, sid) {
                    IsSelected = false
                };
                this.userList.Add(item);
                this.userNodeList.Add(name, item);
                this.userList.Sort();
            }
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                RTFNoteWindow window = new RTFNoteWindow(this);
                ElementHost.EnableModelessKeyboardInterop(window);
                window.Show();
            }
            catch (Exception exception)
            {
                System.Windows.Forms.MessageBox.Show(exception.Message);
            }
        }

        private void bw_DoWork(object sender, DoWorkEventArgs e)
        {
            Action method = null;
            try
            {
                isRunning = true;
                if (method == null)
                {
                    method = () => this.CheckForNewNotes();
                }
                base.Dispatcher.Invoke(method, DispatcherPriority.ApplicationIdle, new object[0]);
                isRunning = false;
            }
            catch (Exception)
            {
                isRunning = false;
            }
        }

        public void Cancel(object sender, RoutedEventArgs e)
        {
            this.CancelButton_Click(sender, e);
        }

        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            this.CloseNote();
        }

        private void CheckForNewNotes()
        {
            try
            {
                if (this.NoteChecked != null)
                {
                    this.NoteChecked(this, null);
                }
            }
            catch (Exception)
            {
                isRunning = false;
            }
        }

        public void Clear()
        {
            if (this._style.Equals("MyNotes"))
            {
                this.MyNotesList.Clear();
            }
            else
            {
                this.ExistingNotesList.Clear();
            }
        }

        private void CloseNote()
        {
            this._noteClosed = true;
            this.NewNoteTextBox.Visibility = Visibility.Visible;
            this.NotesTextbox.Visibility = Visibility.Hidden;
            this.NotesTextbox.Text = this._defaultText;
            this.NotesTextbox.FontStyle = FontStyles.Italic;
            this.NotesTextbox.FontWeight = FontWeights.Bold;
            this.SendToLabel.Visibility = Visibility.Hidden;
            this.SendToComboBox.Visibility = Visibility.Hidden;
            this.NotifyLabel.Visibility = Visibility.Hidden;
            this.NotifyCheckBox.Visibility = Visibility.Hidden;
            this.SaveButton.Visibility = Visibility.Hidden;
            this.CancelButton.Visibility = Visibility.Hidden;
            this.MyNotesList.Visibility = Visibility.Hidden;
            this.ExistingNotesList.Visibility = Visibility.Visible;
            this.AdvancedButton.Visibility = Visibility.Hidden;
            FrameworkElement parent = base.Parent as FrameworkElement;
            if (parent != null)
            {
                base.Width = parent.ActualWidth;
                base.Height = parent.ActualHeight;
                this.NotesTextbox.Width = base.Width;
                this.ExistingNotesList.SetSize(base.Width, base.Height - this.NewNoteTextBox.Height);
            }
            ObservableNodeList itemsSource = (ObservableNodeList) this.SendToComboBox.ItemsSource;
            if (itemsSource != null)
            {
                ObservableNodeList list2 = new ObservableNodeList();
                foreach (Node node2 in itemsSource)
                {
                    Node item = node2;
                    item.IsSelected = false;
                    list2.Add(item);
                }
                this.SendToComboBox.ItemsSource = list2;
            }
        }

        public void CreatePopup(string message, string URL)
        {
            this.uniqueID++;
            NAVPopup.NAVPopup popup = new NAVPopup.NAVPopup {
                PopupText = message,
                URL = URL.Replace(' ', '&'),
                uniqueID = this.uniqueID
            };
            popup.Disposed += new EventHandler(this.note_Disposed);
            this.openPopups.Add(popup);
            popup.position = this.openPopups.Count;
            popup.ShowNote();
        }

        private void EnterNewNote()
        {
            this._noteClosed = false;
            this.NewNoteTextBox.Visibility = Visibility.Hidden;
            this.NotesTextbox.Visibility = Visibility.Visible;
            this.SendToLabel.Visibility = Visibility.Visible;
            this.SendToComboBox.Visibility = Visibility.Visible;
            this.NotifyLabel.Visibility = Visibility.Visible;
            this.NotifyCheckBox.Visibility = Visibility.Visible;
            this.SaveButton.Visibility = Visibility.Visible;
            this.CancelButton.Visibility = Visibility.Visible;
            this.ExistingNotesList.Visibility = Visibility.Visible;
            this.AdvancedButton.Visibility = Visibility.Visible;
            FrameworkElement parent = base.Parent as FrameworkElement;
            if (parent != null)
            {
                base.Width = parent.ActualWidth;
                base.Height = parent.ActualHeight;
                this.ExistingNotesList.SetSize(base.Width, base.Height - ((this.SaveButton.Margin.Top + this.SaveButton.Height) + this.SaveButton.Margin.Bottom));
                this.NotesTextbox.Width = base.Width - 13.0;
            }
            this.MyNotesList.Visibility = Visibility.Hidden;
        }

        public void FinalizeDataLoad()
        {
            this.SendToComboBox.ItemsSource = this.userList;
        }

        [DebuggerNonUserCode]
        public void InitializeComponent()
        {
            if (!this._contentLoaded)
            {
                this._contentLoaded = true;
                Uri resourceLocator = new Uri("/NAVEnhancedNote;component/enhancednotescontrol.xaml", UriKind.Relative);
                System.Windows.Application.LoadComponent(this, resourceLocator);
            }
        }

        private void NewNoteTextBox_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            this.EnterNewNote();
        }

        private void note_Disposed(object sender, EventArgs e)
        {
            int[] numArray = new int[this.openPopups.Count];
            NAVPopup.NAVPopup popup = (NAVPopup.NAVPopup) sender;
            int num = 0;
            int num2 = 0;
            int num3 = 0;
            int index = 0;
            foreach (NAVPopup.NAVPopup popup2 in this.openPopups)
            {
                for (num2 = 0; num2 < num; num2++)
                {
                    popup2.MoveDown();
                }
                if (popup2.uniqueID == popup.uniqueID)
                {
                    numArray[index] = num3;
                    index++;
                    num++;
                }
                num3++;
            }
            for (int i = 0; i < index; i++)
            {
                this.openPopups.RemoveAt(numArray[i]);
            }
        }

        private void NotesTextbox_GotFocus(object sender, RoutedEventArgs e)
        {
            if (this.NotesTextbox.Text == this._defaultText)
            {
                this.NotesTextbox.Text = "";
                this.NotesTextbox.FontStyle = FontStyles.Normal;
                this.NotesTextbox.FontWeight = FontWeights.Normal;
            }
        }

        public void Refresh()
        {
        }

        private void RemoveNote(object sender, EventArgs e)
        {
            this.userFunctionPointer.DynamicInvoke(new object[] { this.linkFromChild });
            object[] args = new object[2];
            this.userControlPointer.DynamicInvoke(args);
        }

        private void Replicate(NAVRecordLink message)
        {
            this.linkFromChild = message;
        }

        private void richTextBox1_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
        {
            this.NewNoteTextBox.TextDecorations = TextDecorations.Underline;
        }

        private void richTextBox1_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
        {
            this.NewNoteTextBox.TextDecorations = null;
        }

        public void Save(object sender, RoutedEventArgs e)
        {
            this.SaveButton_Click(sender, e);
        }

        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            if (this.NoteSaved != null)
            {
                this.NoteSaved(this, e);
            }
            this.CloseNote();
        }

        public void ShowAsMyNotes()
        {
            this.NewNoteTextBox.Visibility = Visibility.Hidden;
            this.NotesTextbox.Visibility = Visibility.Hidden;
            this.SendToLabel.Visibility = Visibility.Hidden;
            this.SendToComboBox.Visibility = Visibility.Hidden;
            this.NotifyLabel.Visibility = Visibility.Hidden;
            this.NotifyCheckBox.Visibility = Visibility.Hidden;
            this.SaveButton.Visibility = Visibility.Hidden;
            this.CancelButton.Visibility = Visibility.Hidden;
            this.ExistingNotesList.Visibility = Visibility.Hidden;
            this.AdvancedButton.Visibility = Visibility.Hidden;
            this.MyNotesList.Visibility = Visibility.Visible;
            FrameworkElement parent = base.Parent as FrameworkElement;
            this.MyNotesList.Width = parent.ActualWidth;
            this.MyNotesList.Height = parent.ActualHeight;
            base.Width = parent.ActualWidth;
            base.Height = parent.ActualHeight;
            this.MyNotesList.SetSize(base.Width, base.Height);
        }

        public void Start()
        {
            BackgroundWorker worker = new BackgroundWorker {
                WorkerReportsProgress = false,
                WorkerSupportsCancellation = false
            };
            this._bw = worker;
            this._bw.DoWork += new DoWorkEventHandler(this.bw_DoWork);
            this.lastRunTime = DateTime.Now;
            this.timer = new System.Windows.Forms.Timer();
            this.timer.Interval = 500;
            this.timer.Tick += new EventHandler(this.timer_Tick);
            this.timer.Start();
            this.openPopups = new ArrayList();
        }

        [EditorBrowsable(EditorBrowsableState.Never), DebuggerNonUserCode]
        void IComponentConnector.Connect(int connectionId, object target)
        {
            switch (connectionId)
            {
                case 1:
                    this.rectangle1 = (Rectangle) target;
                    break;

                case 2:
                    this.NotesTextbox = (System.Windows.Controls.TextBox) target;
                    this.NotesTextbox.LostFocus += new RoutedEventHandler(this.textBox1_LostFocus);
                    this.NotesTextbox.GotFocus += new RoutedEventHandler(this.NotesTextbox_GotFocus);
                    break;

                case 3:
                    this.NotifyCheckBox = (System.Windows.Controls.CheckBox) target;
                    break;

                case 4:
                    this.SendToComboBox = (ComboWithCheckBoxes) target;
                    break;

                case 5:
                    this.NotifyLabel = (System.Windows.Controls.Label) target;
                    break;

                case 6:
                    this.AdvancedButton = (System.Windows.Controls.Button) target;
                    this.AdvancedButton.Click += new RoutedEventHandler(this.button1_Click);
                    break;

                case 7:
                    this.SaveButton = (System.Windows.Controls.Button) target;
                    this.SaveButton.Click += new RoutedEventHandler(this.SaveButton_Click);
                    break;

                case 8:
                    this.CancelButton = (System.Windows.Controls.Button) target;
                    this.CancelButton.Click += new RoutedEventHandler(this.CancelButton_Click);
                    break;

                case 9:
                    this.SendToLabel = (System.Windows.Controls.Label) target;
                    break;

                case 10:
                    this.NewNoteTextBox = (System.Windows.Controls.TextBox) target;
                    this.NewNoteTextBox.MouseEnter += new System.Windows.Input.MouseEventHandler(this.richTextBox1_MouseEnter);
                    this.NewNoteTextBox.MouseLeave += new System.Windows.Input.MouseEventHandler(this.richTextBox1_MouseLeave);
                    this.NewNoteTextBox.MouseLeftButtonDown += new MouseButtonEventHandler(this.NewNoteTextBox_MouseLeftButtonDown);
                    break;

                case 11:
                    this.ExistingNotesList = (NotesListBox) target;
                    break;

                case 12:
                    this.MyNotesList = (MyNotes) target;
                    break;

                default:
                    this._contentLoaded = true;
                    break;
            }
        }

        private void textBox1_LostFocus(object sender, RoutedEventArgs e)
        {
            if (this.NotesTextbox.Text == "")
            {
                this.NotesTextbox.Text = this._defaultText;
                this.NotesTextbox.FontStyle = FontStyles.Italic;
                this.NotesTextbox.FontWeight = FontWeights.Bold;
            }
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            try
            {
                if (!isRunning)
                {
                    this._bw.RunWorkerAsync();
                }
            }
            catch (Exception)
            {
            }
        }

        public string DisplayStyle
        {
            get
            {
                return this._style;
            }
            set
            {
                this._style = value;
                if (this._style.Equals("MyNotes"))
                {
                    this.ShowAsMyNotes();
                }
                else if (this._noteClosed)
                {
                    this.CloseNote();
                }
                else
                {
                    this.EnterNewNote();
                }
                if (this.timer == null)
                {
                    this.Start();
                }
            }
        }

        public string Email
        {
            get
            {
                return this._myEmail;
            }
            set
            {
                this._myEmail = value;
                this.ExistingNotesList.MyEmail = this._myEmail;
                this.MyNotesList.MyEmail = this._myEmail;
            }
        }

        public string Note
        {
            get
            {
                return this.NotesTextbox.Text;
            }
            set
            {
                this.NotesTextbox.Text = value;
            }
        }

        public string Notify
        {
            get
            {
                return (this.NotifyCheckBox.IsChecked.Value ? "yes" : "no");
            }
        }

        public string To
        {
            get
            {
                string str = "";
                ObservableNodeList itemsSource = (ObservableNodeList) this.SendToComboBox.ItemsSource;
                foreach (Node node in itemsSource)
                {
                    if (node.IsSelected)
                    {
                        if (!str.Equals(""))
                        {
                            str = str + ",";
                        }
                        str = str + node.GetGroupMemberLogins();
                    }
                }
                return str;
            }
        }

        public ObservableNodeList UserList
        {
            get
            {
                return this.userList;
            }
            set
            {
                this.userList = value;
            }
        }

        public delegate void CheckForNotes(EnhancedNotesControl m, EventArgs e2);

        public delegate void controlcall(object sender, EventArgs e);

        public delegate void functioncall(NAVRecordLink message);

        public delegate void SaveTheNote(EnhancedNotesControl m, EventArgs e1);
    }
}

