﻿namespace NAVEnhancedNote
{
    using Controls;
    using NodeList;
    using System;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Markup;
    using WpfRichText.Ex.Controls;

    public class RTFNoteWindow : Window, IComponentConnector
    {
        private bool _contentLoaded;
        private EnhancedNotesControl baseControl;
        internal Button CancelButton;
        internal CheckBox NotifyCheckBox;
        internal Label NotifyLabel;
        internal RichTextEditor RTFEditor;
        internal Button SaveButton;
        internal ComboWithCheckBoxes SendToComboBox;
        internal Label SendToLabel;
        private ObservableNodeList userList;

        public RTFNoteWindow(EnhancedNotesControl control)
        {
            this.InitializeComponent();
            this.baseControl = control;
            this.userList = this.baseControl.UserList;
            this.SendToComboBox.ItemsSource = this.userList;
            if (this.baseControl.Note.Equals("Enter a new note here..."))
            {
                this.RTFEditor.Text = "<B><I>Enter a new note here...</I></B>";
            }
            else
            {
                this.RTFEditor.Text = this.baseControl.Note;
            }
            this.NotifyCheckBox.IsChecked = this.baseControl.NotifyCheckBox.IsChecked;
        }

        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            this.baseControl.Cancel(sender, e);
            base.Close();
        }

        [DebuggerNonUserCode]
        public void InitializeComponent()
        {
            if (!this._contentLoaded)
            {
                this._contentLoaded = true;
                Uri resourceLocator = new Uri("/NAVEnhancedNote;component/rtfnotewindow.xaml", UriKind.Relative);
                Application.LoadComponent(this, resourceLocator);
            }
        }

        private void RTFEditor_GotFocus(object sender, RoutedEventArgs e)
        {
            if (this.RTFEditor.Text.Equals("<B><I>Enter a new note here...</I></B>"))
            {
                this.RTFEditor.Text = "";
            }
        }

        private void RTFEditor_LostFocus(object sender, RoutedEventArgs e)
        {
            if (this.RTFEditor.Text.Equals("<HTML><BODY><DIV /></BODY></HTML>"))
            {
                this.RTFEditor.Text = "<B><I>Enter a new note here...</I></B>";
            }
        }

        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            this.baseControl.UserList = this.userList;
            this.baseControl.NotifyCheckBox.IsChecked = this.NotifyCheckBox.IsChecked;
            this.baseControl.Note = this.RTFEditor.Text;
            this.baseControl.Save(sender, e);
            base.Close();
        }

        [EditorBrowsable(EditorBrowsableState.Never), DebuggerNonUserCode]
        void IComponentConnector.Connect(int connectionId, object target)
        {
            switch (connectionId)
            {
                case 1:
                    this.RTFEditor = (RichTextEditor) target;
                    this.RTFEditor.GotFocus += new RoutedEventHandler(this.RTFEditor_GotFocus);
                    this.RTFEditor.LostFocus += new RoutedEventHandler(this.RTFEditor_LostFocus);
                    break;

                case 2:
                    this.SendToLabel = (Label) target;
                    break;

                case 3:
                    this.SendToComboBox = (ComboWithCheckBoxes) target;
                    break;

                case 4:
                    this.NotifyLabel = (Label) target;
                    break;

                case 5:
                    this.NotifyCheckBox = (CheckBox) target;
                    break;

                case 6:
                    this.SaveButton = (Button) target;
                    this.SaveButton.Click += new RoutedEventHandler(this.SaveButton_Click);
                    break;

                case 7:
                    this.CancelButton = (Button) target;
                    this.CancelButton.Click += new RoutedEventHandler(this.CancelButton_Click);
                    break;

                default:
                    this._contentLoaded = true;
                    break;
            }
        }

        public ObservableNodeList UserList
        {
            get
            {
                return this.userList;
            }
        }
    }
}

