﻿namespace NodeList
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.DirectoryServices;
    using System.Runtime.CompilerServices;

    public class Node : IComparable
    {
        private BackgroundWorker _bw;
        private string _groupMembers;
        private User _user = new User();

        public Node(string name, string sid)
        {
            this._user.Name = name;
            this._user.SID = sid;
            this._groupMembers = "Loading...";
            BackgroundWorker worker = new BackgroundWorker {
                WorkerReportsProgress = false,
                WorkerSupportsCancellation = false
            };
            this._bw = worker;
            this._bw.DoWork += new DoWorkEventHandler(this.bw_DoWork);
            this._bw.RunWorkerAsync();
        }

        private void bw_DoWork(object sender, DoWorkEventArgs e)
        {
            this._groupMembers = this.GetGroupMembers();
        }

        public int CompareTo(object o)
        {
            Node node = (Node) o;
            return string.Compare(this.Title, node.Title);
        }

        public string GetGroupMemberLogins()
        {
            string str = "";
            DirectoryEntry entry = new DirectoryEntry(string.Format("LDAP://<SID={0}>", this._user.SID));
            PropertyCollection properties = entry.Properties;
            bool flag = false;
            List<string> list = new List<string>();
            foreach (object obj2 in properties["member"])
            {
                flag = true;
                DirectoryEntry entry2 = new DirectoryEntry("LDAP://" + obj2.ToString());
                PropertyCollection propertys2 = entry2.Properties;
                foreach (object obj3 in entry2.Properties["sAMAccountName"])
                {
                    list.Add(obj3.ToString());
                }
            }
            if (!flag)
            {
                foreach (object obj3 in properties["sAMAccountName"])
                {
                    list.Add(obj3.ToString());
                }
            }
            list.Sort();
            foreach (string str2 in list)
            {
                if (!str.Equals(""))
                {
                    str = str + ",";
                }
                str = str + str2;
            }
            return str;
        }

        public string GetGroupMembers()
        {
            string str = "";
            DirectoryEntry entry = new DirectoryEntry(string.Format("LDAP://<SID={0}>", this._user.SID));
            PropertyCollection properties = entry.Properties;
            bool flag = false;
            List<string> list = new List<string>();
            foreach (object obj2 in properties["member"])
            {
                flag = true;
                DirectoryEntry entry2 = new DirectoryEntry("LDAP://" + obj2.ToString());
                foreach (object obj3 in entry2.Properties["CN"])
                {
                    list.Add(obj3.ToString());
                }
            }
            if (!flag)
            {
                foreach (object obj3 in properties["CN"])
                {
                    list.Add(obj3.ToString());
                }
            }
            list.Sort();
            foreach (string str2 in list)
            {
                if (!str.Equals(""))
                {
                    str = str + Environment.NewLine;
                }
                str = str + str2;
            }
            return str;
        }

        public string GroupMembers
        {
            get
            {
                return this._groupMembers;
            }
        }

        public string ID
        {
            get
            {
                return this._user.SID;
            }
            set
            {
                this._user.SID = value;
            }
        }

        public bool IsSelected { get; set; }

        public string Title
        {
            get
            {
                return this._user.Name;
            }
            set
            {
                this._user.Name = value;
            }
        }

        public User user
        {
            get
            {
                return this._user;
            }
        }
    }
}

