﻿namespace NodeList
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Collections.Specialized;
    using System.Text;
    using System.Threading;
    using System.Windows.Threading;

    public class ObservableNodeList : ObservableCollection<Node>
    {
        public event NotifyCollectionChangedEventHandler CollectionChanged;

        protected override void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
        {
            using (base.BlockReentrancy())
            {
                NotifyCollectionChangedEventHandler collectionChanged = this.CollectionChanged;
                if (collectionChanged != null)
                {
                    Delegate[] invocationList = collectionChanged.GetInvocationList();
                    foreach (NotifyCollectionChangedEventHandler handler2 in invocationList)
                    {
                        DispatcherObject target = handler2.Target as DispatcherObject;
                        if (!((target == null) || target.CheckAccess()))
                        {
                            target.Dispatcher.Invoke(DispatcherPriority.DataBind, handler2, this, new object[] { e });
                        }
                        else
                        {
                            handler2(this, e);
                        }
                    }
                }
            }
        }

        public void Sort()
        {
            this.Sort(Comparer<Node>.Default);
        }

        public void Sort(IComparer<Node> comparer)
        {
            for (int i = 1; i < base.Count; i++)
            {
                Node y = base[i];
                int num2 = i;
                while ((num2 > 0) && (comparer.Compare(base[num2 - 1], y) == 1))
                {
                    base[num2] = base[num2 - 1];
                    num2--;
                }
                base[num2] = y;
            }
        }

        public override string ToString()
        {
            StringBuilder builder = new StringBuilder();
            foreach (Node node in base.Items)
            {
                if (node.IsSelected)
                {
                    builder.Append(node.Title);
                    builder.Append(',');
                }
            }
            return builder.ToString().TrimEnd(new char[] { ',' });
        }
    }
}

