﻿namespace WpfRichText.Ex.AttachedProperties
{
    using System;
    using System.IO;
    using System.Text;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;
    using System.Windows.Documents;
    using System.Windows.Markup;
    using WpfRichText.Ex.XamlToHtmlParser;

    public static class RichTextboxAssistant
    {
        public static readonly DependencyProperty BoundDocument = DependencyProperty.RegisterAttached("BoundDocument", typeof(string), typeof(RichTextboxAssistant), new FrameworkPropertyMetadata(null, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault, new PropertyChangedCallback(RichTextboxAssistant.OnBoundDocumentChanged)));

        private static void AttachEventHandler(RichTextBox box)
        {
            Binding binding = BindingOperations.GetBinding(box, BoundDocument);
            if (binding != null)
            {
                if ((binding.UpdateSourceTrigger == UpdateSourceTrigger.Default) || (binding.UpdateSourceTrigger == UpdateSourceTrigger.LostFocus))
                {
                    box.LostFocus += new RoutedEventHandler(RichTextboxAssistant.HandleLostFocus);
                }
                else
                {
                    box.TextChanged += new TextChangedEventHandler(RichTextboxAssistant.HandleTextChanged);
                }
            }
        }

        public static string GetBoundDocument(DependencyObject dp)
        {
            string str = dp.GetValue(BoundDocument) as string;
            string str2 = string.Empty;
            if (!string.IsNullOrEmpty(str))
            {
                str2 = HtmlToXamlConverter.ConvertHtmlToXaml(str, false);
            }
            return str2;
        }

        private static void HandleLostFocus(object sender, RoutedEventArgs e)
        {
            RichTextBox dp = sender as RichTextBox;
            TextRange range = new TextRange(dp.Document.ContentStart, dp.Document.ContentEnd);
            using (MemoryStream stream = new MemoryStream())
            {
                range.Save(stream, DataFormats.Xaml);
                string str = Encoding.Default.GetString(stream.ToArray());
                SetBoundDocument(dp, str);
            }
        }

        private static void HandleTextChanged(object sender, RoutedEventArgs e)
        {
            RichTextBox dp = sender as RichTextBox;
            TextRange range = new TextRange(dp.Document.ContentStart, dp.Document.ContentEnd);
            using (MemoryStream stream = new MemoryStream())
            {
                range.Save(stream, DataFormats.Xaml);
                string str = Encoding.Default.GetString(stream.ToArray());
                SetBoundDocument(dp, str);
            }
        }

        private static void OnBoundDocumentChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            RichTextBox box = d as RichTextBox;
            if (box != null)
            {
                RemoveEventHandler(box);
                string boundDocument = GetBoundDocument(d);
                box.Document.Blocks.Clear();
                if (!string.IsNullOrEmpty(boundDocument))
                {
                    using (MemoryStream stream = new MemoryStream(Encoding.ASCII.GetBytes(boundDocument)))
                    {
                        ParserContext parserContext = new ParserContext();
                        parserContext.XmlnsDictionary.Add("", "http://schemas.microsoft.com/winfx/2006/xaml/presentation");
                        parserContext.XmlnsDictionary.Add("x", "http://schemas.microsoft.com/winfx/2006/xaml");
                        FlowDocument document = new FlowDocument();
                        Section item = XamlReader.Load(stream, parserContext) as Section;
                        box.Document.Blocks.Add(item);
                    }
                }
                AttachEventHandler(box);
            }
        }

        private static void RemoveEventHandler(RichTextBox box)
        {
            Binding binding = BindingOperations.GetBinding(box, BoundDocument);
            if (binding != null)
            {
                if ((binding.UpdateSourceTrigger == UpdateSourceTrigger.Default) || (binding.UpdateSourceTrigger == UpdateSourceTrigger.LostFocus))
                {
                    box.LostFocus -= new RoutedEventHandler(RichTextboxAssistant.HandleLostFocus);
                }
                else
                {
                    box.TextChanged -= new TextChangedEventHandler(RichTextboxAssistant.HandleTextChanged);
                }
            }
        }

        public static void SetBoundDocument(DependencyObject dp, string value)
        {
            string xamlString = value;
            string str2 = HtmlFromXamlConverter.ConvertXamlToHtml(xamlString, false);
            dp.SetValue(BoundDocument, str2);
        }
    }
}

