﻿namespace WpfRichText.Ex.Commands
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Input;

    internal class CommandManagerHelper
    {
        internal static void AddHandlersToRequerySuggested(List<WeakReference> handlers)
        {
            if (handlers != null)
            {
                foreach (WeakReference reference in handlers)
                {
                    EventHandler target = reference.Target as EventHandler;
                    if (target != null)
                    {
                        CommandManager.RequerySuggested += target;
                    }
                }
            }
        }

        internal static void AddWeakReferenceHandler(ref List<WeakReference> handlers, EventHandler handler)
        {
            AddWeakReferenceHandler(ref handlers, handler, -1);
        }

        internal static void AddWeakReferenceHandler(ref List<WeakReference> handlers, EventHandler handler, int defaultListSize)
        {
            if (handlers == null)
            {
                handlers = (defaultListSize > 0) ? new List<WeakReference>(defaultListSize) : new List<WeakReference>();
            }
            handlers.Add(new WeakReference(handler));
        }

        internal static void CallWeakReferenceHandlers(List<WeakReference> handlers)
        {
            if (handlers != null)
            {
                int num2;
                EventHandler target;
                EventHandler[] handlerArray = new EventHandler[handlers.Count];
                int index = 0;
                for (num2 = handlers.Count - 1; num2 >= 0; num2--)
                {
                    WeakReference reference = handlers[num2];
                    target = reference.Target as EventHandler;
                    if (target == null)
                    {
                        handlers.RemoveAt(num2);
                    }
                    else
                    {
                        handlerArray[index] = target;
                        index++;
                    }
                }
                for (num2 = 0; num2 < index; num2++)
                {
                    target = handlerArray[num2];
                    target(null, EventArgs.Empty);
                }
            }
        }

        internal static void RemoveHandlersFromRequerySuggested(List<WeakReference> handlers)
        {
            if (handlers != null)
            {
                foreach (WeakReference reference in handlers)
                {
                    EventHandler target = reference.Target as EventHandler;
                    if (target != null)
                    {
                        CommandManager.RequerySuggested -= target;
                    }
                }
            }
        }

        internal static void RemoveWeakReferenceHandler(List<WeakReference> handlers, EventHandler handler)
        {
            if (handlers != null)
            {
                for (int i = handlers.Count - 1; i >= 0; i--)
                {
                    WeakReference reference = handlers[i];
                    EventHandler target = reference.Target as EventHandler;
                    if ((target == null) || (target == handler))
                    {
                        handlers.RemoveAt(i);
                    }
                }
            }
        }
    }
}

