﻿namespace WpfRichText.Ex.Commands
{
    using System;
    using System.Threading;
    using System.Windows;
    using System.Windows.Input;

    public class CommandReference : Freezable, ICommand
    {
        public static readonly DependencyProperty CommandProperty = DependencyProperty.Register("Command", typeof(ICommand), typeof(CommandReference), new PropertyMetadata(new PropertyChangedCallback(CommandReference.OnCommandChanged)));

        public event EventHandler CanExecuteChanged;

        public bool CanExecute(object parameter)
        {
            return ((this.Command != null) && this.Command.CanExecute(parameter));
        }

        protected override Freezable CreateInstanceCore()
        {
            throw new NotImplementedException();
        }

        public void Execute(object parameter)
        {
            this.Command.Execute(parameter);
        }

        private static void OnCommandChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            CommandReference reference = d as CommandReference;
            ICommand oldValue = e.OldValue as ICommand;
            ICommand newValue = e.NewValue as ICommand;
            if (oldValue != null)
            {
                oldValue.CanExecuteChanged -= reference.CanExecuteChanged;
            }
            if (newValue != null)
            {
                newValue.CanExecuteChanged += reference.CanExecuteChanged;
            }
        }

        public ICommand Command
        {
            get
            {
                return (ICommand) base.GetValue(CommandProperty);
            }
            set
            {
                base.SetValue(CommandProperty, value);
            }
        }
    }
}

