﻿namespace WpfRichText.Ex.XamlToHtmlParser
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.Text;

    internal class HtmlLexicalAnalyzer
    {
        private bool _ignoreNextWhitespace;
        private StringReader _inputStringReader;
        private bool _isNextCharacterEntity;
        private char _lookAheadCharacter;
        private int _lookAheadCharacterCode;
        private char _nextCharacter;
        private int _nextCharacterCode;
        private StringBuilder _nextToken;
        private HtmlTokenType _nextTokenType;
        private char _previousCharacter;

        internal HtmlLexicalAnalyzer(string inputTextString)
        {
            this._inputStringReader = new StringReader(inputTextString);
            this._nextCharacterCode = 0;
            this._nextCharacter = ' ';
            this._lookAheadCharacterCode = this._inputStringReader.Read();
            this._lookAheadCharacter = (char) this._lookAheadCharacterCode;
            this._previousCharacter = ' ';
            this._ignoreNextWhitespace = true;
            this._nextToken = new StringBuilder(100);
            this._nextTokenType = HtmlTokenType.Text;
            this.GetNextCharacter();
        }

        internal void GetNextAtomToken()
        {
            Debug.Assert(this._nextTokenType != HtmlTokenType.EOF);
            this._nextToken.Length = 0;
            this.SkipWhiteSpace();
            this._nextTokenType = HtmlTokenType.Atom;
            if (((this.NextCharacter != '\'') && (this.NextCharacter != '"')) || this.IsNextCharacterEntity)
            {
                while ((!this.IsAtEndOfStream && !char.IsWhiteSpace(this.NextCharacter)) && (this.NextCharacter != '>'))
                {
                    this._nextToken.Append(this.NextCharacter);
                    this.GetNextCharacter();
                }
                return;
            }
            char nextCharacter = this.NextCharacter;
            this.GetNextCharacter();
        Label_007C:;
            if (((this.NextCharacter != nextCharacter) || this.IsNextCharacterEntity) && !this.IsAtEndOfStream)
            {
                this._nextToken.Append(this.NextCharacter);
                this.GetNextCharacter();
                goto Label_007C;
            }
            if (this.NextCharacter == nextCharacter)
            {
                this.GetNextCharacter();
            }
        }

        private void GetNextCharacter()
        {
            if (this._nextCharacterCode == -1)
            {
                throw new InvalidOperationException("GetNextCharacter method called at the end of a stream");
            }
            this._previousCharacter = this._nextCharacter;
            this._nextCharacter = this._lookAheadCharacter;
            this._nextCharacterCode = this._lookAheadCharacterCode;
            this._isNextCharacterEntity = false;
            this.ReadLookAheadCharacter();
            if (this._nextCharacter == '&')
            {
                int num2;
                if (this._lookAheadCharacter == '#')
                {
                    int num = 0;
                    this.ReadLookAheadCharacter();
                    for (num2 = 0; (num2 < 7) && char.IsDigit(this._lookAheadCharacter); num2++)
                    {
                        num = (10 * num) + (this._lookAheadCharacterCode - 0x30);
                        this.ReadLookAheadCharacter();
                    }
                    if (this._lookAheadCharacter == ';')
                    {
                        this.ReadLookAheadCharacter();
                        this._nextCharacterCode = num;
                        this._nextCharacter = (char) this._nextCharacterCode;
                        this._isNextCharacterEntity = true;
                    }
                    else
                    {
                        this._nextCharacter = this._lookAheadCharacter;
                        this._nextCharacterCode = this._lookAheadCharacterCode;
                        this.ReadLookAheadCharacter();
                        this._isNextCharacterEntity = false;
                    }
                }
                else if (char.IsLetter(this._lookAheadCharacter))
                {
                    string entityName = "";
                    for (num2 = 0; (num2 < 10) && (char.IsLetter(this._lookAheadCharacter) || char.IsDigit(this._lookAheadCharacter)); num2++)
                    {
                        entityName = entityName + this._lookAheadCharacter;
                        this.ReadLookAheadCharacter();
                    }
                    if (this._lookAheadCharacter == ';')
                    {
                        this.ReadLookAheadCharacter();
                        if (HtmlSchema.IsEntity(entityName))
                        {
                            this._nextCharacter = HtmlSchema.EntityCharacterValue(entityName);
                            this._nextCharacterCode = this._nextCharacter;
                            this._isNextCharacterEntity = true;
                        }
                        else
                        {
                            this._nextCharacter = this._lookAheadCharacter;
                            this._nextCharacterCode = this._lookAheadCharacterCode;
                            this.ReadLookAheadCharacter();
                            this._isNextCharacterEntity = false;
                        }
                    }
                    else
                    {
                        this._nextCharacter = this._lookAheadCharacter;
                        this.ReadLookAheadCharacter();
                        this._isNextCharacterEntity = false;
                    }
                }
            }
        }

        internal void GetNextContentToken()
        {
            Debug.Assert(this._nextTokenType != HtmlTokenType.EOF);
            this._nextToken.Length = 0;
            if (this.IsAtEndOfStream)
            {
                this._nextTokenType = HtmlTokenType.EOF;
            }
            else if (this.IsAtTagStart)
            {
                this.GetNextCharacter();
                if (this.NextCharacter == '/')
                {
                    this._nextToken.Append("</");
                    this._nextTokenType = HtmlTokenType.ClosingTagStart;
                    this.GetNextCharacter();
                    this._ignoreNextWhitespace = false;
                }
                else
                {
                    this._nextTokenType = HtmlTokenType.OpeningTagStart;
                    this._nextToken.Append("<");
                    this._ignoreNextWhitespace = true;
                }
            }
            else if (this.IsAtDirectiveStart)
            {
                this.GetNextCharacter();
                if (this._lookAheadCharacter == '[')
                {
                    this.ReadDynamicContent();
                }
                else if (this._lookAheadCharacter == '-')
                {
                    this.ReadComment();
                }
                else
                {
                    this.ReadUnknownDirective();
                }
            }
            else
            {
                this._nextTokenType = HtmlTokenType.Text;
                while ((!this.IsAtTagStart && !this.IsAtEndOfStream) && !this.IsAtDirectiveStart)
                {
                    if (((this.NextCharacter == '<') && !this.IsNextCharacterEntity) && (this._lookAheadCharacter == '?'))
                    {
                        this.SkipProcessingDirective();
                    }
                    else
                    {
                        if (this.NextCharacter <= ' ')
                        {
                            if (!this._ignoreNextWhitespace)
                            {
                                this._nextToken.Append(' ');
                            }
                            this._ignoreNextWhitespace = true;
                        }
                        else
                        {
                            this._nextToken.Append(this.NextCharacter);
                            this._ignoreNextWhitespace = false;
                        }
                        this.GetNextCharacter();
                    }
                }
            }
        }

        internal void GetNextEqualSignToken()
        {
            Debug.Assert(this._nextTokenType != HtmlTokenType.EOF);
            this._nextToken.Length = 0;
            this._nextToken.Append('=');
            this._nextTokenType = HtmlTokenType.EqualSign;
            this.SkipWhiteSpace();
            if (this.NextCharacter == '=')
            {
                this.GetNextCharacter();
            }
        }

        internal void GetNextTagToken()
        {
            this._nextToken.Length = 0;
            if (this.IsAtEndOfStream)
            {
                this._nextTokenType = HtmlTokenType.EOF;
            }
            else
            {
                this.SkipWhiteSpace();
                if (!((this.NextCharacter != '>') || this.IsNextCharacterEntity))
                {
                    this._nextTokenType = HtmlTokenType.TagEnd;
                    this._nextToken.Append('>');
                    this.GetNextCharacter();
                }
                else if ((this.NextCharacter == '/') && (this._lookAheadCharacter == '>'))
                {
                    this._nextTokenType = HtmlTokenType.EmptyTagEnd;
                    this._nextToken.Append("/>");
                    this.GetNextCharacter();
                    this.GetNextCharacter();
                    this._ignoreNextWhitespace = false;
                }
                else if (this.IsGoodForNameStart(this.NextCharacter))
                {
                    this._nextTokenType = HtmlTokenType.Name;
                    while (this.IsGoodForName(this.NextCharacter) && !this.IsAtEndOfStream)
                    {
                        this._nextToken.Append(this.NextCharacter);
                        this.GetNextCharacter();
                    }
                }
                else
                {
                    this._nextTokenType = HtmlTokenType.Atom;
                    this._nextToken.Append(this.NextCharacter);
                    this.GetNextCharacter();
                }
            }
        }

        private bool IsCombiningCharacter(char character)
        {
            return false;
        }

        private bool IsExtender(char character)
        {
            return false;
        }

        private bool IsGoodForName(char character)
        {
            return ((((this.IsGoodForNameStart(character) || (character == '.')) || ((character == '-') || (character == ':'))) || (char.IsDigit(character) || this.IsCombiningCharacter(character))) || this.IsExtender(character));
        }

        private bool IsGoodForNameStart(char character)
        {
            return ((character == '_') || char.IsLetter(character));
        }

        private void ReadComment()
        {
            bool flag;
            Debug.Assert(((this._previousCharacter == '<') && (this._nextCharacter == '!')) && (this._lookAheadCharacter == '-'));
            this._nextTokenType = HtmlTokenType.Comment;
            this._nextToken.Length = 0;
            this.GetNextCharacter();
            this.GetNextCharacter();
            this.GetNextCharacter();
        Label_0123:
            flag = true;
            while (!this.IsAtEndOfStream && (((this._nextCharacter != '-') || (this._lookAheadCharacter != '-')) && ((this._nextCharacter != '!') || (this._lookAheadCharacter != '>'))))
            {
                this._nextToken.Append(this.NextCharacter);
                this.GetNextCharacter();
            }
            this.GetNextCharacter();
            if (((this._previousCharacter == '-') && (this._nextCharacter == '-')) && (this._lookAheadCharacter == '>'))
            {
                this.GetNextCharacter();
            }
            else if ((this._previousCharacter != '!') || (this._nextCharacter != '>'))
            {
                this._nextToken.Append(this._previousCharacter);
                goto Label_0123;
            }
            if (this._nextCharacter == '>')
            {
                this.GetNextCharacter();
            }
        }

        private void ReadDynamicContent()
        {
            Debug.Assert(((this._previousCharacter == '<') && (this._nextCharacter == '!')) && (this._lookAheadCharacter == '['));
            this._nextTokenType = HtmlTokenType.Text;
            this._nextToken.Length = 0;
            this.GetNextCharacter();
            this.GetNextCharacter();
        Label_0055:;
            if (((this._nextCharacter != ']') || (this._lookAheadCharacter != '>')) && !this.IsAtEndOfStream)
            {
                this.GetNextCharacter();
                goto Label_0055;
            }
            if (!this.IsAtEndOfStream)
            {
                this.GetNextCharacter();
                this.GetNextCharacter();
            }
        }

        private void ReadLookAheadCharacter()
        {
            if (this._lookAheadCharacterCode != -1)
            {
                this._lookAheadCharacterCode = this._inputStringReader.Read();
                this._lookAheadCharacter = (char) this._lookAheadCharacterCode;
            }
        }

        private void ReadUnknownDirective()
        {
            Debug.Assert(((this._previousCharacter == '<') && (this._nextCharacter == '!')) && ((this._lookAheadCharacter != '-') && (this._lookAheadCharacter != '[')));
            this._nextTokenType = HtmlTokenType.Text;
            this._nextToken.Length = 0;
            this.GetNextCharacter();
        Label_005E:;
            if (((this._nextCharacter != '>') || this.IsNextCharacterEntity) && !this.IsAtEndOfStream)
            {
                this.GetNextCharacter();
                goto Label_005E;
            }
            if (!this.IsAtEndOfStream)
            {
                this.GetNextCharacter();
            }
        }

        private void SkipProcessingDirective()
        {
            Debug.Assert((this._nextCharacter == '<') && (this._lookAheadCharacter == '?'));
            this.GetNextCharacter();
            this.GetNextCharacter();
        Label_0037:;
            if ((((this._nextCharacter != '?') && (this._nextCharacter != '/')) || (this._lookAheadCharacter != '>')) && !this.IsAtEndOfStream)
            {
                this.GetNextCharacter();
                goto Label_0037;
            }
            if (!this.IsAtEndOfStream)
            {
                this.GetNextCharacter();
                this.GetNextCharacter();
            }
        }

        private void SkipWhiteSpace()
        {
            while (true)
            {
                if ((this._nextCharacter == '<') && ((this._lookAheadCharacter == '?') || (this._lookAheadCharacter == '!')))
                {
                    this.GetNextCharacter();
                    if (this._lookAheadCharacter != '[')
                    {
                        while (!this.IsAtEndOfStream && (this._nextCharacter != '>'))
                        {
                            this.GetNextCharacter();
                        }
                        if (this._nextCharacter == '>')
                        {
                            this.GetNextCharacter();
                        }
                    }
                    else
                    {
                        while (!this.IsAtEndOfStream && (((this._previousCharacter != ']') || (this._nextCharacter != ']')) || (this._lookAheadCharacter != '>')))
                        {
                            this.GetNextCharacter();
                        }
                        if (this._nextCharacter == '>')
                        {
                            this.GetNextCharacter();
                        }
                    }
                }
                if (!char.IsWhiteSpace(this.NextCharacter))
                {
                    return;
                }
                this.GetNextCharacter();
            }
        }

        private bool IsAtDirectiveStart
        {
            get
            {
                return (((this._nextCharacter == '<') && (this._lookAheadCharacter == '!')) && !this.IsNextCharacterEntity);
            }
        }

        private bool IsAtEndOfStream
        {
            get
            {
                return (this._nextCharacterCode == -1);
            }
        }

        private bool IsAtTagEnd
        {
            get
            {
                return (((this._nextCharacter == '>') || ((this._nextCharacter == '/') && (this._lookAheadCharacter == '>'))) && !this._isNextCharacterEntity);
            }
        }

        private bool IsAtTagStart
        {
            get
            {
                return (((this._nextCharacter == '<') && ((this._lookAheadCharacter == '/') || this.IsGoodForNameStart(this._lookAheadCharacter))) && !this._isNextCharacterEntity);
            }
        }

        private bool IsNextCharacterEntity
        {
            get
            {
                return this._isNextCharacterEntity;
            }
        }

        private char NextCharacter
        {
            get
            {
                return this._nextCharacter;
            }
        }

        internal string NextToken
        {
            get
            {
                return this._nextToken.ToString();
            }
        }

        internal HtmlTokenType NextTokenType
        {
            get
            {
                return this._nextTokenType;
            }
        }
    }
}

